<?php
/**
 * Plugin Name:       Argus | Social Media WordPress Plugin
 * Plugin URI:        https://www.persianscript.ir
 * Description:       Argus is a WordPress plugin that connects your website with different social platforms. Currently, it supports integration with Discord and Steam. Twitch support is coming next, and I plan to extend it to other APIs.
 * Version:           6.0.11
 * Requires at least: 6.3
 * Requires PHP:      7.4
 * Author:            پرشین اسکریپت
 * Author URI:        https://www.persianscript.ir
 * Text Domain:       argus-wordpress-plugin
 */

if (!defined('ABSPATH')) exit;


// Defining constants
const ARGUS_PLUGIN_VERSION    = '6.0.11';
const ARGUS_ELEMENTOR_VERSION = '3.16';
const ARGUS_ENVATO_ID         = '32050244';
const ARGUS_ENVATO_LINK       = 'https://codecanyon.net/item/argus-discord-wordpress-plugin/32050244';
const ARGUS_DOC_LINK          = 'https://mykolapetrovych.gitbook.io/argus-wordpress-plugin/';
const ARGUS_SURVEY_LINK       = 'https://forms.gle/BWT1M4rtLgeER2Yr5';
const ARGUS_SUPPORT_EMAIL     = 'support@mykolapetrovych.com';

define('ARGUS_PLUGIN_PATH', plugin_dir_path(__FILE__));
define('ARGUS_PLUGIN_URL', plugin_dir_url(__FILE__));
define('ARGUS_ACTIVE_COMPONENTS', explode(',', get_option('argusActiveComponents')));

if (in_array('elementor/elementor.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    define('ARGUS_ELEMENTOR_ACTIVE', true);
} else {
    define('ARGUS_ELEMENTOR_ACTIVE', false);
}


// Loading models
include_once ARGUS_PLUGIN_PATH . 'models/API.php';
include_once ARGUS_PLUGIN_PATH . 'models/Data.php';
include_once ARGUS_PLUGIN_PATH . 'models/Log.php';

// Loading controllers
include_once ARGUS_PLUGIN_PATH . 'controllers/Uninstall.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/Activate.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/Deactivate.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/Cron.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/ajax-hooks.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/AdminSettingsAbstraction.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/ComponentsPanel.php';
include_once ARGUS_PLUGIN_PATH . 'controllers/Update.php';

// Loading sub-plugins
include_once ARGUS_PLUGIN_PATH . 'components/discord/discord.php';
include_once ARGUS_PLUGIN_PATH . 'components/steam/steam.php';


// Checking if structure update is required
if (!wp_doing_ajax() && get_option('argusLastUpdateOnVersion') != ARGUS_PLUGIN_VERSION) {
    Argus\Update::update();
}


// Registering translations
add_action('plugins_loaded', function() {
    if ((int)get_option('argusActivateTranslations')) {
        // NOTE: Po file should have name like: argus-wordpress-plugin-uk.po
        load_plugin_textdomain('argus-wordpress-plugin', false, 'argus-wordpress-plugin/languages/');
    }
});


// Registering plugin installation method
register_activation_hook(__FILE__, ['ARGUS\Activate', 'activate']);

// Registering plugin deactivation method
register_deactivation_hook(__FILE__, ['ARGUS\Deactivate', 'deactivate']);

// Registering plugin removing method
register_uninstall_hook(__FILE__, ['ARGUS\Uninstall', 'uninstall']);


// Initializing cron
Argus\Cron::init();


// Saving last valid address for redirect after API authorization
add_action('wp_footer', ['Argus\Data', 'saveURLForRedirect'], 100, 0);
add_action('admin_footer', ['Argus\Data', 'saveURLForRedirect'], 100, 0);


// Registering admin panel libraries
add_action('admin_enqueue_scripts', function() {
    wp_register_script('argus-admin-helpers-scripts', ARGUS_PLUGIN_URL . 'controllers/admin-helpers.js', ['jquery'], ARGUS_PLUGIN_VERSION, true);
});

// Admin panel actions
if (is_admin()) {

    // Registering plugin installation UI controls
    $pluginMainFile = plugin_basename(__FILE__);
    add_filter("plugin_action_links_$pluginMainFile", ['ARGUS\Activate', 'pluginInstallationUIControls'], 1, 1);
    add_filter("network_admin_plugin_action_$pluginMainFile", ['ARGUS\Activate', 'pluginInstallationUIControls'], 1, 1);

    // Registering admin panel pages
    add_action('admin_menu', function() {

        // Registering assets
        wp_register_style('argus-admin-panel-styles', ARGUS_PLUGIN_URL . 'views/styles/admin-panel.css', [], ARGUS_PLUGIN_VERSION);

        // Registering admin panel controls
        $page = add_submenu_page('plugins.php', __("Argus Components", 'argus-wordpress-plugin'), __("Argus Components", 'argus-wordpress-plugin'), 'install_plugins', 'argus-components-and-settings', ['Argus\ComponentsPanel', 'init']);

        add_action('load-' . $page, function() {
            wp_enqueue_style('argus-admin-panel-styles');
            wp_enqueue_script('argus-components-panel', ARGUS_PLUGIN_URL . 'controllers/ComponentsPanel.js', ['jquery', 'react', 'react-dom'], ARGUS_PLUGIN_VERSION, true);
            wp_enqueue_script('argus-admin-helpers-scripts');
        });
    });

    // Registering update controls
    add_action('core_upgrade_preamble', ['Argus\Update', 'renderUpdateControls']);
    Argus\Update::checkForUpdateCounter();

    // Old plugin installed notice
    Argus\Update::subscribeOldPluginNotice();
}
