<?php

namespace Argus\Discord;

use Elementor\Plugin;


class AccountPages {

    static string $woocommerceSlug        = 'woocommerce-discord';
    static string $ultimateMemberSlug     = 'ultimate-member-discord';
    static int    $ultimateMemberPosition = 1000;
    static string $memberpressSlug        = 'memberpress-discord';
    static string $buddypressSlug         = 'buddypress-discord';


    static function init () {

        if (get_option('argusDiscordWoocommerceAccountPage')) {
            add_filter('woocommerce_account_menu_items', ['Argus\Discord\AccountPages', 'addWoocommerceMenuItem'], 100, 1);
            add_filter('woocommerce_get_endpoint_url', ['Argus\Discord\AccountPages', 'addWoocommercePageURL'], 100, 2);
            add_filter('init', ['Argus\Discord\AccountPages', 'addWoocommerceAccountEndpoint'], 100, 0);
            add_filter('woocommerce_account_' . self::$woocommerceSlug . '_endpoint', ['Argus\Discord\AccountPages', 'addWoocommerceAccountEndpointContent'], 100, 0);
        }

        if (get_option('argusDiscordUltimateMemberAccountPage')) {
            add_filter('um_account_page_default_tabs_hook', ['Argus\Discord\AccountPages', 'addUltimateMemberAccountTab'], 100, 1);
            add_filter('um_account_content_hook_' . self::$ultimateMemberSlug, ['Argus\Discord\AccountPages', 'addUltimateMemberAccountEndpointContent'], 20, 2);
        }

        if (get_option('argusDiscordMemberPressAccountPage')) {
            add_action('mepr_account_nav', ['Argus\Discord\AccountPages', 'addMemberPressAccountTab'], 100, 0);
            add_action('mepr_account_nav_content', ['Argus\Discord\AccountPages', 'addMemberPressAccountEndpointContent'], 100, 1);
        }

        if (get_option('argusDiscordBuddypressAccountPage') && function_exists('bp_core_new_nav_item') && function_exists('bp_core_load_template')) {
            add_action('bp_setup_nav', ['Argus\Discord\AccountPages', 'addBuddyPressMenuItem'], 1, 0);
        }

    }


    static function addBuddyPressMenuItem () {
        $pageID = get_option('argusDiscordBuddypressAccountPage');

        bp_core_new_nav_item([
            'name'            => get_the_title($pageID),
            'slug'            => self::$buddypressSlug,
            'position'        => 100,
            'screen_function' => ['Argus\Discord\AccountPages', 'addBuddyPressAccountEndpoint']
        ]);
    }


    static function addBuddyPressAccountEndpoint () {
        add_action('bp_template_title', ['Argus\Discord\AccountPages', 'addBuddyPressAccountPageTitle']);
        add_action('bp_template_content', ['Argus\Discord\AccountPages', 'addBuddyPressAccountPageContent']);
        bp_core_load_template(apply_filters('bp_core_template_plugin', 'members/single/plugins'));
    }


    static function addBuddyPressAccountPageTitle () {
        $pageID = get_option('argusDiscordBuddypressAccountPage');
        echo get_the_title($pageID);
    }


    static function addBuddyPressAccountPageContent () {
        self::echoPageContent(get_option('argusDiscordBuddypressAccountPage'));
    }


    static function addWoocommerceMenuItem ($menuLinks) {
        $pageID = get_option('argusDiscordWoocommerceAccountPage');

        $new = [get_post_field('post_name', $pageID) => get_the_title($pageID)];
        return array_slice($menuLinks, 0, 1, true) + $new + array_slice($menuLinks, 1, null, true);
    }


    static function addWoocommercePageURL ($url, $endpoint) {
        $pageID = get_option('argusDiscordWoocommerceAccountPage');

        if ($endpoint === get_post_field('post_name', $pageID)) {
            $url = get_permalink(get_option('woocommerce_myaccount_page_id')) . self::$woocommerceSlug;
        }

        return $url;
    }


    static function addWoocommerceAccountEndpoint () {
        add_rewrite_endpoint(self::$woocommerceSlug, EP_PAGES);
    }


    static function addWoocommerceAccountEndpointContent () {
        self::echoPageContent(get_option('argusDiscordWoocommerceAccountPage'));
    }


    static function addUltimateMemberAccountTab ($tabs):array {
        $pageID = get_option('argusDiscordUltimateMemberAccountPage');
        $icon   = get_option('argusDiscordUltimateMemberAccountPageIcon');

        $tabs[ self::$ultimateMemberPosition ][ self::$ultimateMemberSlug ] = [
            'icon'        => $icon ?: 'um-faicon-link',
            'title'       => get_the_title($pageID),
            'custom'      => true,
            'show_button' => false,
        ];

        return $tabs;
    }


    static function addUltimateMemberAccountEndpointContent ($output = '', $args = []):string {
        ob_start();
        self::echoPageContent(get_option('argusDiscordUltimateMemberAccountPage'));
        $output .= ob_get_clean();
        return $output;
    }


    static function addMemberPressAccountTab () {
        if (defined('MEPR_OPTIONS_SLUG') && class_exists('MeprAccountHelper')) {

            $memberpressOptions = get_option(MEPR_OPTIONS_SLUG);
            if (isset($memberpressOptions['account_page_id'])) {
                $slug   = get_post_field('post_name', $memberpressOptions['account_page_id']);
                $pageID = get_option('argusDiscordMemberPressAccountPage');

                ?>
                <span class="mepr-nav-item <?php MeprAccountHelper::active_nav(self::$memberpressSlug); ?>">
                    <a href="/<?php echo $slug; ?>/?action=<?php echo self::$memberpressSlug; ?>">
                        <?php echo get_the_title($pageID); ?>
                    </a>
                </span>
                <?php

            }
        }
    }


    static function addMemberPressAccountEndpointContent ($action) {
        if ($action == self::$memberpressSlug) {
            self::echoPageContent(get_option('argusDiscordMemberPressAccountPage'));
        }
    }


    private static function echoPageContent ($pageID) {
        if ($pageID) {
            $page = get_post($pageID);
            if (isset($page->post_content)) {
                if (get_post_meta($page->ID, '_elementor_template_type') && class_exists('\Elementor\Plugin')) {
                    $pluginElementor = Plugin::instance();
                    echo $pluginElementor->frontend->get_builder_content($pageID);
                } else {
                    echo do_shortcode(do_blocks($page->post_content));
                }
            }
        }
    }

}
