<?php

namespace Argus\Discord;

use Argus\AdminSettingsAbstraction;


class AdminSettings extends AdminSettingsAbstraction {

    protected static string $lastError;


    static function init () {
        // Loading errors
        self::$lastError = Data::getLastError();

        // Pages routing
        if (!get_option('argusDiscordOAuth2AccessToken')) {
            self::displayAuthorisationPage();
        } else {
            self::displayMainSettingsPage();
        }
    }


    static function displayAuthorisationPage () {
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/authorisation-fields.php';
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/authorisation.phtml';
    }


    static function displayMainSettingsPage () {
        wp_enqueue_script('argus-discord-admin-panel-scripts', ARGUS_DISCORD_PLUGIN_URL . 'controllers/AdminSettings.js', ['jquery', 'react', 'react-dom'], ARGUS_PLUGIN_VERSION, true);

        global $wp_roles;
        $data = new Data();

        $connectedGuild   = Data::getData('Guild');
        $dynamicColors    = self::buildDynamicAdminColors();
        $translationsFile = ARGUS_DISCORD_PLUGIN_PATH . '/languages/argus-discord-react-' . get_locale() . '.json';
        $allRoleBindings  = $data->getAllRoleBindingsList();
        $discordRoles     = $data->getAllRolesList();
        $connectedApp     = Data::getData('App');
        $accountAvatar    = $data->getAvatarLinkOfTheGuildMember($connectedApp->owner, 128);
        $registeredCPT    = get_post_types([], 'object');
        $appAvatar        = $data->getAppIconLink($connectedApp, 128);
        $publishedPages   = get_pages([
            'post_status' => 'publish',
            'sort_column' => 'title',
            'sort_order'  => 'desc'
        ]);

        ?>
        <script>
            var argusStateValues = {};

            <?php if ((int)get_option('argusActivateTranslations') && file_exists($translationsFile)) { ?>
            argusStateValues.translations = <?php echo file_get_contents($translationsFile); ?>;
            <?php } else { ?>
            argusStateValues.translations = false;
            <?php } ?>

            <?php if (isset($_COOKIE['argusInitUpdate']) && $_COOKIE['argusInitUpdate']) { ?>
            document.cookie             = 'argusInitUpdate=0; expires=0; path=/';
            argusStateValues.initUpdate = true;
            <?php } else { ?>
            argusStateValues.initUpdate = false;
            <?php } ?>

            argusStateValues.colors = {
                accent: "<?php echo $dynamicColors[2]['bg']; ?>",
                text  : "<?php echo $dynamicColors[2]['text']; ?>"
            }

            argusStateValues.criticalErrorsNotification = "<?php echo addslashes(self::checkForCriticalErrors(1)); ?>";
            argusStateValues.elementorError             = "<?php echo addslashes(self::checkElementorVersion()); ?>";
            argusStateValues.supportEmail               = "<?php echo ARGUS_SUPPORT_EMAIL; ?>";
            argusStateValues.supportExpireDate          = "<?php echo self::buildSupportDateTemplate(); ?>";
            argusStateValues.documentationLink          = "<?php echo ARGUS_DOC_LINK; ?>";
            argusStateValues.accountUsername            = "<?php echo addslashes($connectedApp->owner->username); ?>";
            argusStateValues.accountAvatar              = "<?php echo esc_url($accountAvatar); ?>";
            argusStateValues.appName                    = "<?php echo addslashes($connectedApp->name); ?>";
            argusStateValues.appAvatar                  = "<?php echo esc_url($appAvatar); ?>";
            argusStateValues.appID                      = "<?php echo esc_html($connectedApp->id); ?>";
            argusStateValues.logoutURL                  = "<?php echo admin_url('options-general.php?page=argus-discord-plugin-settings&argus-logout=1'); ?>";
            argusStateValues.webhookName                = "<?php echo addslashes($data->getWebhookChannelName()); ?>";
            argusStateValues.guildID                    = "<?php echo $connectedGuild->id ?? ''; ?>";
            argusStateValues.guildName                  = "<?php echo addslashes($connectedGuild->name ?? ''); ?>";
            argusStateValues.optionsPermalinkURL        = "<?php echo get_admin_url(null, 'options-permalink.php'); ?>";
            argusStateValues.ultimateMemberIconsNote    = "<?php echo self::checkAndBuildUltimateMemberIconLibraries(); ?>";

            argusStateValues.registeredCPT = [
                <?php foreach($registeredCPT as $cpt) {if ($cpt->name != 'revision' && $cpt->name != 'nav_menu_item') { ?>
                {
                    value: "<?php echo $cpt->name; ?>",
                    name : "<?php echo addslashes($cpt->label); ?>"
                },
                <?php }} ?>
            ];

            argusStateValues.publishedPages = [
                <?php foreach($publishedPages as $page) { ?>
                {
                    value: "<?php echo $page->ID; ?>",
                    name : "<?php echo esc_html($page->post_title); ?>"
                },
                <?php } ?>
            ];

            argusStateValues.allRoleBindings = [
                <?php foreach($allRoleBindings as $binding) { ?>
                {
                    <?php if (isset($wp_roles->roles[ $binding->wp_role ])) { ?>
                    wp    : "<?php echo $binding->wp_role; ?>",
                    wpName: "<?php echo esc_html($wp_roles->roles[ $binding->wp_role ]['name']); ?>",
                    <?php } else { ?>
                    wp    : 0,
                    wpName: "[<?php _e("warning, role was removed", 'argus-wordpress-plugin'); ?>]",
                    <?php } ?>
                    direction  : <?php echo $binding->direction; ?>,
                    discord    : "<?php echo $binding->discord_role; ?>",
                    discordName: "<?php echo esc_html($binding->discord_role_name); ?>"
                },
                <?php } ?>
            ];

            argusStateValues.wpRoles = [
                <?php foreach($wp_roles->roles as $key => $value) { ?>
                {
                    value: "<?php echo $key; ?>",
                    name : "<?php echo esc_html($value['name']); ?>"
                },
                <?php } ?>
            ];

            argusStateValues.discordRoles = [
                <?php foreach($discordRoles as $role) {if ($role->name != '@everyone') { ?>
                {
                    value: "<?php echo $role->id; ?>",
                    name : "<?php echo esc_html($role->name); ?>"
                },
                <?php }}?>
            ];

            argusStateValues.settingsValues = {
                argusDiscordAnnouncementCPT                     : "<?php echo get_option('argusDiscordAnnouncementCPT'); ?>",
                argusDiscordAnnouncementAddEveryone             : <?php echo (int)get_option('argusDiscordAnnouncementAddEveryone'); ?>,
                argusDiscordAnnouncementAddExcerpt              : <?php echo (int)get_option('argusDiscordAnnouncementAddExcerpt'); ?>,
                argusDiscordAnnouncementAddDate                 : <?php echo (int)get_option('argusDiscordAnnouncementAddDate'); ?>,
                argusDiscordAnnouncementUpdates                 : <?php echo (int)get_option('argusDiscordAnnouncementUpdates'); ?>,
                argusDiscordCommentsCPT                         : "<?php echo get_option('argusDiscordCommentsCPT'); ?>",
                argusDiscordStylingOAuthButtons                 : <?php echo (int)get_option('argusDiscordStylingOAuthButtons'); ?>,
                argusDiscordModifyWPLoginForm                   : <?php echo (int)get_option('argusDiscordModifyWPLoginForm'); ?>,
                argusDiscordModifyMemberPressLoginForm          : <?php echo (int)get_option('argusDiscordModifyMemberPressLoginForm'); ?>,
                argusDiscordDeleteFromGuildIfProfileDisconnected: <?php echo (int)get_option('argusDiscordDeleteFromGuildIfProfileDisconnected'); ?>,
                argusDiscordBlockNotVerified                    : <?php echo (int)get_option('argusDiscordBlockNotVerified'); ?>,
                argusDiscordBlockPending                        : <?php echo (int)get_option('argusDiscordBlockPending'); ?>,
                argusDiscordGuildFilterMode                     : "<?php echo get_option('argusDiscordGuildFilterMode'); ?>",
                argusDiscordGuildFilter                         : "<?php echo get_option('argusDiscordGuildFilter'); ?>",
                argusDiscordRolesFilterMode                     : "<?php echo get_option('argusDiscordRolesFilterMode'); ?>",
                argusDiscordRolesFilter                         : "<?php echo get_option('argusDiscordRolesFilter'); ?>",
                argusDiscordFilterProfileConnection             : <?php echo (int)get_option('argusDiscordFilterProfileConnection'); ?>,
                argusDiscordNewUserRedirect                     : "<?php echo get_option('argusDiscordNewUserRedirect'); ?>",
                argusDiscordNewUserRedirectInNewTab             : <?php echo (int)get_option('argusDiscordNewUserRedirectInNewTab'); ?>,
                argusDiscordNewUserRedirectIfNotInGuild         : <?php echo (int)get_option('argusDiscordNewUserRedirectIfNotInGuild'); ?>,
                argusDiscordProfileConnectedRedirect            : "<?php echo get_option('argusDiscordProfileConnectedRedirect'); ?>",
                argusDiscordCustomRedirects                     : [
                    <?php $redirectArray = json_decode(get_option('argusDiscordCustomRedirects')); ?>
                    <?php if ($redirectArray) {foreach($redirectArray as $id => $redirect) { ?>
                    {
                        id  : <?php echo $id; ?>,
                        name: "<?php echo $redirect->name; ?>",
                        url : "<?php echo $redirect->url; ?>"
                    },
                    <?php }} ?>
                    {
                        id  : <?php echo time(); ?>,
                        name: '',
                        url : ''
                    }],
                argusDiscordWoocommerceAccountPage              : "<?php echo get_option('argusDiscordWoocommerceAccountPage'); ?>",
                argusDiscordAssignRoleIfProfileConnected        : "<?php echo get_option('argusDiscordAssignRoleIfProfileConnected'); ?>",
                argusDiscordUltimateMemberAccountPage           : "<?php echo get_option('argusDiscordUltimateMemberAccountPage'); ?>",
                argusDiscordUltimateMemberAccountPageIcon       : "<?php echo get_option('argusDiscordUltimateMemberAccountPageIcon'); ?>",
                argusDiscordMemberPressAccountPage              : "<?php echo get_option('argusDiscordMemberPressAccountPage'); ?>",
                argusDiscordBuddypressAccountPage               : "<?php echo get_option('argusDiscordBuddypressAccountPage'); ?>",
                argusDiscordReplaceWPAvatars                    : <?php echo (int)get_option('argusDiscordReplaceWPAvatars'); ?>,
                argusDiscordReplaceBuddypressAvatars            : <?php echo (int)get_option('argusDiscordReplaceBuddypressAvatars'); ?>,
                argusDiscordDefaultAvatar                       : <?php echo (int)get_option('argusDiscordDefaultAvatar'); ?>,
                argusDiscordDefaultAvatarURL                    : "<?php echo Avatars::getDefaultAvatarUrl([64, 64]); ?>"
            };
        </script>

        <div id="argus-discord-admin-settings"></div>
        <?php
    }


    static function modifyProfilePage ($user) {
        $discordID   = get_user_meta($user->ID, 'dwpp-discord-id', true);
        $profileData = Data::findProfile($discordID);
        $displayName = self::buildDisplayName($profileData);

        wp_enqueue_script('argus-admin-helpers-scripts');
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/profile-settings.phtml';
    }


    static function modifyPersonalProfilePage ($user) {
        $discordID   = get_user_meta($user->ID, 'dwpp-discord-id', true);
        $connected   = false;
        $displayName = '';

        if ($discordID) {
            $profileData = Data::findProfile($discordID);
            $displayName = self::buildDisplayName($profileData);
            $connected   = true;
        }
        $authorizationLink = Data::getAuthorizationLink('argus-connect-discord-profile-to-user');

        wp_enqueue_script('argus-admin-helpers-scripts');
        include_once ARGUS_DISCORD_PLUGIN_PATH . 'views/includes/personal-profile-settings.phtml';
    }


    static function saveDataFromProfilePage ($userID) {
        if (!current_user_can('edit_user', $userID)) return false;
        $newID = '';
        if (isset($_POST['dwpp-discord-id']) && intval($_POST['dwpp-discord-id'])) {
            $newID = wp_strip_all_tags($_POST['dwpp-discord-id']);
        }
        return update_user_meta($userID, 'dwpp-discord-id', $newID);
    }


    static function modifyPluginsManagerPage ($links) {
        $settingsLink = '<a href="' . admin_url('options-general.php?page=argus-discord-plugin-settings') . '">' . __("Settings", 'argus-wordpress-plugin') . '</a>';
        array_unshift($links, $settingsLink);
        return $links;
    }


    private static function checkAndBuildUltimateMemberIconLibraries ():string {
        $fa     = $ii = false;
        $faName = 'um-fonticons-fa.css';
        $iiName = 'um-fonticons-ii.css';
        $faPath = '/ultimate-member/assets/css/' . $faName;
        $iiPath = '/ultimate-member/assets/css/' . $iiName;
        if (file_exists(WP_PLUGIN_DIR . $faPath)) $fa = true;
        if (file_exists(WP_PLUGIN_DIR . $iiPath)) $ii = true;

        $noteText = __("Here, you must write icon class from Ultimate Member icon styles.", 'argus-wordpress-plugin');
        if ($fa || $ii) {
            $noteText .= '<br>';
            $noteText .= __("Official documentation of Ultimate Member does not have any icon references. Still, these icons are found in the installed version of the plugin:", 'argus-wordpress-plugin');
            if ($fa) {
                $noteText .= "&nbsp;<a href='" . plugins_url() . $faPath . "' target='_blank' style='white-space: nowrap'>" . $faName . '</a>';
            }
            if ($ii) {
                $noteText .= "&nbsp;<a href='" . plugins_url() . $iiPath . "' target='_blank' style='white-space: nowrap'>" . $iiName . '</a>';
            }
            $noteText .= '<br>';
            $noteText .= __("Maybe Ultimate Member is loading other icon sets that were not found. You may ask Ultimate Member support.", 'argus-wordpress-plugin');
        }

        return $noteText;
    }


    private static function buildDisplayName ($profileData) {
        $displayName = '';
        if (isset($profileData->username) && $profileData->username) $displayName = $profileData->username;
        if (isset($profileData->discriminator) && $profileData->discriminator) {
            if (isset($profileData->username) && $profileData->username) $displayName .= '#';
            $displayName .= $profileData->username;
        }
        return $displayName;
    }

}
