<?php

namespace Argus\Discord;

use WP_User;

class RolesSync extends Data {


    static function syncAllUsersRoles ():bool {
        $linkedUsers = get_users([
            'meta_key'     => 'dwpp-discord-id',
            'meta_value'   => ' ',
            'meta_compare' => '!=',
            'orderby'      => 'login',
            'order'        => 'ASC',
            'fields'       => ['ID'],
        ]);

        if (count($linkedUsers)) {
            foreach ($linkedUsers as $user) {
                if (!self::syncUserRoles($user->ID)) return false;
            }
        }
        return true;
    }


    static function syncUserRoles ($wpUserID, $filterDirection = false):bool {
        $_SESSION['argus_running_user_roles_synchronization'] = true;

        // Getting bindings info
        $bindings = self::getAllRoleBindingsList();
        if (!count($bindings)) {
            unset($_SESSION['argus_running_user_roles_synchronization']);
            return true;
        }

        // Getting connected discord profile
        $discordUserID = get_user_meta($wpUserID, 'dwpp-discord-id', true);
        if (!$discordUserID) {
            unset($_SESSION['argus_running_user_roles_synchronization']);
            return true;
        }

        // Getting wp user roles
        $wpUserRoles = get_userdata($wpUserID)->roles;

        // Getting discord user roles if direction requires
        $discordUserRoles = false;
        foreach ($bindings as $binding) {
            if ($binding->direction == 2) {
                $discordUserRoles = self::getSingleMemberData($discordUserID);
                if (!$discordUserRoles || !isset($discordUserRoles->roles)) {
                    unset($_SESSION['argus_running_user_roles_synchronization']);
                    return false;
                }
                $discordUserRoles = $discordUserRoles->roles;
                break;
            }
        }

        // Cycling each bindings
        foreach ($bindings as $binding) {

            // If syncing only in one direction
            if ($filterDirection && $binding->direction != $filterDirection) continue;

            // If syncing from wp to discord
            if ($binding->direction == 1) {

                // Defining if user has wp role from the binding rule
                $role_found = false;
                foreach ($wpUserRoles as $wpRole) {
                    if ($wpRole == $binding->wp_role) {
                        $role_found = true;
                        break;
                    }
                }

                // If user has required wp role
                if ($role_found) {
                    if (!self::addUserRole($discordUserID, $binding->discord_role, $wpUserID)) {
                        unset($_SESSION['argus_running_user_roles_synchronization']);
                        return false;
                    }

                    // If user has no required wp role
                } elseif (!self::removeUserRole($discordUserID, $binding->discord_role, $wpUserID)) {
                    unset($_SESSION['argus_running_user_roles_synchronization']);
                    return false;
                }

                // If syncing from discord to wp
            } elseif ($binding->direction == 2) {

                // If no discord roles was received skip to next binding
                if (isset($discordUserRoles->roles) && !count($discordUserRoles->roles)) continue;

                // Defining if user has discord role from the binding rule
                $role_found = false;
                foreach ($discordUserRoles as $discordRole) {
                    if ($discordRole == $binding->discord_role) {
                        $role_found = true;
                        break;
                    }
                }

                // If user has required discord role
                if ($role_found) {
                    (new WP_User($wpUserID))->add_role($binding->wp_role);

                    // If user has no required discord role
                } else {
                    (new WP_User($wpUserID))->remove_role($binding->wp_role);
                }
            }
        }

        // Check if at the end wp user has no roles
        if (!count(get_userdata($wpUserID)->roles)) {
            (new WP_User($wpUserID))->set_role(get_option('default_role'));
        }

        unset($_SESSION['argus_running_user_roles_synchronization']);
        return true;
    }


    static function syncWPUserRoles ($user_login, $user) {
        if (get_user_meta($user->data->ID, 'dwpp-discord-id', true)) {
            RolesSync::syncUserRoles($user->data->ID);
        }
    }


    static function syncAuthorizedWPUserData () {
        $wpUserID      = get_current_user_id();
        $discordUserID = get_user_meta($wpUserID, 'dwpp-discord-id', true);

        if ($discordUserID) {
            Data::fetchSingleMemberData($discordUserID, true);
        }

        if ($wpUserID) RolesSync::syncUserRoles($wpUserID);

        wp_redirect($_COOKIE['argusLastPage'] ?? '/');
        exit();
    }


    static function syncWPUserRolesAfterRoleChange ($userID) {
        if (!isset($_SESSION['argus_running_user_roles_synchronization']) && get_user_meta($userID, 'dwpp-discord-id', true)) {
            RolesSync::syncUserRoles($userID, 1);
        }
    }


}
