<?php

if (!defined('ABSPATH')) exit;


add_action('wp_ajax_argusDiscordFetchAuthorizedProfile', 'argusDiscordFetchAuthorizedProfile');
add_action('wp_ajax_nopriv_argusDiscordFetchAuthorizedProfile', 'argusDiscordFetchAuthorizedProfile');
function argusDiscordFetchAuthorizedProfile () {
    if (!Argus\Discord\Data::fetchAuthorizedProfile()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Profile data updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchAuthorizedApp', 'argusDiscordFetchAuthorizedApp');
add_action('wp_ajax_nopriv_argusDiscordFetchAuthorizedApp', 'argusDiscordFetchAuthorizedApp');
function argusDiscordFetchAuthorizedApp () {
    if (!Argus\Discord\Data::fetchAuthorizedApp()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Bot data updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchGuild', 'argusDiscordFetchGuild');
add_action('wp_ajax_nopriv_argusDiscordFetchGuild', 'argusDiscordFetchGuild');
function argusDiscordFetchGuild () {
    if (!Argus\Discord\Data::fetchGuild()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Guild data updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchMembers', 'argusDiscordFetchMembers');
add_action('wp_ajax_nopriv_argusDiscordFetchMembers', 'argusDiscordFetchMembers');
function argusDiscordFetchMembers () {
    if (!Argus\Discord\Data::fetchMembers()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Members data updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchAllRoles', 'argusDiscordFetchAllRoles');
add_action('wp_ajax_nopriv_argusDiscordFetchAllRoles', 'argusDiscordFetchAllRoles');
function argusDiscordFetchAllRoles () {
    if (!Argus\Discord\Data::fetchAllRoles()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        $discordRoles     = Argus\Discord\Data::getAllRolesList();
        $discordRolesJson = [];
        foreach ($discordRoles as $role) {
            if ($role->name != '@everyone') {
                $discordRolesJson[] = [
                    'value' => $role->id,
                    'name'  => esc_html($role->name),
                ];
            }
        }
        wp_send_json_success([
            'message' => __("Roles data updated", 'argus-wordpress-plugin'),
            'state'   => ['discordRoles' => $discordRolesJson]
        ]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchAllChannels', 'argusDiscordFetchAllChannels');
add_action('wp_ajax_nopriv_argusDiscordFetchAllChannels', 'argusDiscordFetchAllChannels');
function argusDiscordFetchAllChannels () {
    if (!Argus\Discord\Data::fetchAllChannels()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Channels data & invite links are updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchSingleChannel', 'argusDiscordFetchSingleChannel');
add_action('wp_ajax_nopriv_argusDiscordFetchSingleChannel', 'argusDiscordFetchSingleChannel');
function argusDiscordFetchSingleChannel () {
    $data = new Argus\Discord\Data();
    $id   = wp_strip_all_tags($_POST['id']);
    if (!$data->fetchSingleChannel($id)) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        $message = $data->getChannelName($id) . ' ' . __("channel data are updated", 'argus-wordpress-plugin');
        wp_send_json_success(['message' => $message]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchMessagesFromAllChannels', 'argusDiscordFetchMessagesFromAllChannels');
add_action('wp_ajax_nopriv_argusDiscordFetchMessagesFromAllChannels', 'argusDiscordFetchMessagesFromAllChannels');
function argusDiscordFetchMessagesFromAllChannels () {
    if (!Argus\Discord\Data::fetchMessagesFromAllChannels()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Messages updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchPinnedMessagesFromAllChannels', 'argusDiscordFetchPinnedMessagesFromAllChannels');
add_action('wp_ajax_nopriv_argusDiscordFetchPinnedMessagesFromAllChannels', 'argusDiscordFetchPinnedMessagesFromAllChannels');
function argusDiscordFetchPinnedMessagesFromAllChannels () {
    if (!Argus\Discord\Data::fetchPinnedMessagesFromAllChannels()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Pinned messages updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordGetInterpretedMessages', 'argusDiscordGetInterpretedMessages');
add_action('wp_ajax_nopriv_argusDiscordGetInterpretedMessages', 'argusDiscordGetInterpretedMessages');
function argusDiscordGetInterpretedMessages () {
    $interpretedMessages = [];
    $_GET['pinned']      = isset($_GET['pinned']) && $_GET['pinned'] == 'true';

    $messages = Argus\Discord\Data::getAllMessagesData(wp_strip_all_tags($_GET['channel']), intval($_GET['quantity']), $_GET['pinned']);


    $filter = [
        'customEmoji' => $_GET['customEmoji'] == 'true',
        'images'      => $_GET['images'] == 'true',
        'links'       => $_GET['links'] == 'true',
        'tenor'       => $_GET['tenor'] == 'true',
    ];
    foreach ($messages as $message) {
        $interpretedMessages[] = Argus\Discord\Data::interpretMessage($message, $filter);
    }

    wp_send_json_success(['message' => $interpretedMessages]);
    exit;
}


add_action('wp_ajax_argusDiscordSyncAllUsersRoles', 'argusDiscordSyncAllUsersRoles');
add_action('wp_ajax_nopriv_argusDiscordSyncAllUsersRoles', 'argusDiscordSyncAllUsersRoles');
function argusDiscordSyncAllUsersRoles () {
    if (!Argus\Discord\RolesSync::syncAllUsersRoles()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Roles synchronized", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordSyncUserRoles', 'argusDiscordSyncUserRoles');
add_action('wp_ajax_nopriv_argusDiscordSyncUserRoles', 'argusDiscordSyncUserRoles');
function argusDiscordSyncUserRoles () {
    if (!Argus\Discord\RolesSync::syncUserRoles(wp_strip_all_tags($_POST['id']))) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        $message = get_user_by('id', wp_strip_all_tags($_POST['id']))->data->display_name . ' ' . __("roles synchronized", 'argus-wordpress-plugin');
        wp_send_json_success(['message' => $message]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordFetchAllChannelsInviteLinks', 'argusDiscordFetchAllChannelsInviteLinks');
add_action('wp_ajax_nopriv_argusDiscordFetchAllChannelsInviteLinks', 'argusDiscordFetchAllChannelsInviteLinks');
function argusDiscordFetchAllChannelsInviteLinks () {
    if (!Argus\Discord\Data::fetchAllChannelsInviteLinks()) {
        wp_send_json_error(['message' => Argus\Discord\Data::getLastError()]);
    } else {
        wp_send_json_success(['message' => __("Invite links are updated", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordSendMessage', 'argusDiscordSendMessage');
add_action('wp_ajax_nopriv_argusDiscordSendMessage', 'argusDiscordSendMessage');
function argusDiscordSendMessage () {

    $response = Argus\Discord\Data::sendMessage(wp_strip_all_tags($_GET['text']), false, ($_GET['tts'] == 'true' ? 1 : 0), wp_strip_all_tags($_GET['image']));

    if (isset($response->message)) {
        wp_send_json_error($response->message);
    } elseif (isset($response->error)) wp_send_json_error($response->error);
    else wp_send_json_success(__("Message Sent", 'argus-wordpress-plugin'));
}


add_action('wp_ajax_argusDiscordGetImage', 'argusDiscordGetImage');
add_action('wp_ajax_nopriv_argusDiscordGetImage', 'argusDiscordGetImage');
function argusDiscordGetImage () {
    if (isset($_GET['id'])) {
        $image = wp_get_attachment_image(filter_input(INPUT_GET, 'id', FILTER_VALIDATE_INT), wp_strip_all_tags($_GET['size']));
        wp_send_json_success(['image' => $image]);

    } else {
        wp_send_json_error();
    }
}


add_action('wp_ajax_argusDiscordGetConnectedGuildId', 'argusDiscordGetConnectedGuildId');
add_action('wp_ajax_nopriv_argusDiscordGetConnectedGuildId', 'argusDiscordGetConnectedGuildId');
function argusDiscordGetConnectedGuildId () {
    wp_send_json_success(['id' => get_option('argusDiscordGuild')]);
    exit;
}


add_action('wp_ajax_argusDiscordGetAllChannelsList', 'argusDiscordGetAllChannelsList');
add_action('wp_ajax_nopriv_argusDiscordGetAllChannelsList', 'argusDiscordGetAllChannelsList');
function argusDiscordGetAllChannelsList () {
    wp_send_json_success(['channels' => Argus\Discord\Data::getAllChannelsList()]);
    exit;
}


add_action('wp_ajax_argusDiscordGetMembersCount', 'argusDiscordGetMembersCount');
add_action('wp_ajax_nopriv_argusDiscordGetMembersCount', 'argusDiscordGetMembersCount');
function argusDiscordGetMembersCount () {
    wp_send_json_success(['count' => Argus\Discord\Data::getData('Guild')->approximate_member_count]);
    exit;
}


add_action('wp_ajax_argusDiscordAddRolesSync', 'argusDiscordAddRolesSync');
add_action('wp_ajax_nopriv_argusDiscordAddRolesSync', 'argusDiscordAddRolesSync');
function argusDiscordAddRolesSync () {
    if (Argus\Discord\Data::addRoleBinding($_POST['argus_wp_role'], $_POST['argus_discord_role'], $_POST['argus_direction'])) {
        wp_send_json_success();
    } else {
        wp_send_json_error(['message' => __("Option not updated, error on the server...", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordDeleteRolesSync', 'argusDiscordDeleteRolesSync');
add_action('wp_ajax_nopriv_argusDiscordDeleteRolesSync', 'argusDiscordDeleteRolesSync');
function argusDiscordDeleteRolesSync () {
    if (Argus\Discord\Data::deleteRoleBinding($_POST['argus_direction'], $_POST['argus_wp_role'], $_POST['argus_discord_role'])) {
        wp_send_json_success();
    } else {
        wp_send_json_error(['message' => __("Option not updated, error on the server...", 'argus-wordpress-plugin')]);
    }
    exit;
}


add_action('wp_ajax_argusDiscordAllDiscordDataUpdateList', 'argusDiscordAllDiscordDataUpdateList');
add_action('wp_ajax_nopriv_argusDiscordAllDiscordDataUpdateList', 'argusDiscordAllDiscordDataUpdateList');
function argusDiscordAllDiscordDataUpdateList () {
    $data = new Argus\Discord\Data();
    if (!$data->fetchAllChannels()) {
        wp_send_json_error(__("Can not get access to list of channels...", 'argus-wordpress-plugin'));
        exit;
    }
    $channels    = $data->getAllChannelsData();
    $linkedUsers = get_users([
        'meta_key'     => 'dwpp-discord-id',
        'meta_value'   => ' ',
        'meta_compare' => '!=',
        'orderby'      => 'login',
        'order'        => 'ASC',
        'fields'       => ['ID', 'display_name'],
    ]);

    $json = [
        [
            'text'  => __("Downloading connected profile data...", 'argus-wordpress-plugin'),
            'data'  => [
                'action' => 'argusDiscordFetchAuthorizedProfile'
            ],
            'state' => 4
        ],
        [
            'text'  => __("Downloading connected Discord bot data...", 'argus-wordpress-plugin'),
            'data'  => [
                'action' => 'argusDiscordFetchAuthorizedApp'
            ],
            'state' => 4
        ],
        [
            'text'  => __("Downloading guild data...", 'argus-wordpress-plugin'),
            'data'  => [
                'action' => 'argusDiscordFetchGuild'
            ],
            'state' => 4
        ],
        [
            'text'  => __("Downloading guild members data...", 'argus-wordpress-plugin'),
            'data'  => [
                'action' => 'argusDiscordFetchMembers'
            ],
            'state' => 4
        ],
        [
            'text'  => __("Downloading guild roles data...", 'argus-wordpress-plugin'),
            'data'  => [
                'action' => 'argusDiscordFetchAllRoles'
            ],
            'state' => 4
        ]
    ];

    if (count($channels)) {
        foreach ($channels as $channel) {
            $json[] = [
                'text'  => __("Downloading data from channel", 'argus-wordpress-plugin') . ' ' . esc_html($channel->object->name) . '...',
                'data'  => [
                    'action' => 'argusDiscordFetchSingleChannel',
                    'id'     => $channel->id
                ],
                'state' => 4
            ];
        }
    }

    if (count($linkedUsers)) {
        foreach ($linkedUsers as $user) {
            $json[] = [
                'text'  => __("Syncing user", 'argus-wordpress-plugin') . ' ' . esc_html($user->display_name) . '...',
                'data'  => [
                    'action' => 'argusDiscordSyncUserRoles',
                    'id'     => $user->ID
                ],
                'state' => 4
            ];
        }
    }

    wp_send_json_success($json);
    exit;
}
