<?php

namespace Argus\Discord;

class API {

    private static int $sleepRequestTolerance = 20;
    private static int $forceRequestRetryTime = 15;

    private static array $lastCurlHeaders = [];


    protected static function requestAuthorizedProfileData ($keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/users/@me', $keys['argusDiscordOAuth2AccessToken'], 'Bearer');
    }


    protected static function requestAuthorizedAppData ($keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/oauth2/applications/@me', $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function requestToAddUserRole ($keys, $guild, $user, $role) {
        $url  = 'https://discordapp.com/api/guilds/' . $guild . '/members/' . $user . '/roles/' . $role;
        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_URL, $url);

        curl_setopt($curl, CURLOPT_PUT, true);
        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Authorization: Bot ' . $keys['argusDiscordOAuth2BotToken'],
            'Content-Type: application/json'
        ]);
        self::$lastCurlHeaders = [];
        curl_setopt($curl, CURLOPT_HEADERFUNCTION, ['Argus\Discord\API', 'parseHeadersResponse']);

        $response     = json_decode(curl_exec($curl));
        $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if (intval($responseCode) == 429) {
            sleep(self::getSleepTime($response));
            return self::requestToAddUserRole($keys, $guild, $user, $role);
        }

        self::$lastCurlHeaders = [];
        return $response;
    }


    protected static function requestToRemoveUserRole ($keys, $guild, $user, $role) {
        $url = 'https://discordapp.com/api/guilds/' . $guild . '/members/' . $user . '/roles/' . $role;

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_URL, $url);

        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'DELETE');
        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Authorization: Bot ' . $keys['argusDiscordOAuth2BotToken'],
            'Content-Type: application/json'
        ]);
        self::$lastCurlHeaders = [];
        curl_setopt($curl, CURLOPT_HEADERFUNCTION, ['Argus\Discord\API', 'parseHeadersResponse']);

        $response     = json_decode(curl_exec($curl));
        $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if (intval($responseCode) == 429) {
            sleep(self::getSleepTime($response));
            return self::requestToRemoveUserRole($keys, $guild, $user, $role);
        }

        self::$lastCurlHeaders = [];
        return $response;
    }


    protected static function requestToRemoveUserFromServer ($keys, $guild, $discordUserID) {
        $url = 'https://discordapp.com/api/guilds/' . $guild . '/members/' . $discordUserID;

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_URL, $url);

        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Authorization: Bot ' . $keys['argusDiscordOAuth2BotToken'],
            "Content-Type: application/json",
            "X-Audit-Log-Reason: The user disconnected Discord profile on the website"
        ]);

        $response = json_decode(curl_exec($curl));
        curl_close($curl);

        return $response;
    }


    protected static function requestAllGuildsUserAreMemberOf ($keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/users/@me/guilds', $keys['argusDiscordOAuth2AccessToken'], 'Bearer');
    }


    protected static function requestAllRoles ($guild, $keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/guilds/' . $guild . '/roles', $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function requestAllChannels ($guild, $keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/guilds/' . $guild . '/channels', $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function requestChannelInvites ($channel, $keys) {
        $curl = curl_init('https://discordapp.com/api/channels/' . $channel . '/invites');

        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode(['max_age' => 0, 'max_uses' => 0, 'temporary' => false, 'xkcdpass' => false], JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Accept: application/json',
            'Authorization: Bot ' . $keys['argusDiscordOAuth2BotToken'],
        ]);
        self::$lastCurlHeaders = [];
        curl_setopt($curl, CURLOPT_HEADERFUNCTION, ['Argus\Discord\API', 'parseHeadersResponse']);

        $response     = json_decode(curl_exec($curl));
        $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if (intval($responseCode) == 429) {
            sleep(self::getSleepTime($response));
            return self::requestChannelInvites($channel, $keys);
        }

        self::$lastCurlHeaders = [];
        return $response;
    }


    protected static function requestGuildData ($guild, $keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/guilds/' . $guild . '?with_counts=true', $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function requestMembersData ($guild, $keys, $limit = 1000, $after = 0) {
        $afterStr = '';
        if ($after) $afterStr = '&after=' . $after;

        $curl = curl_init('https://discordapp.com/api/guilds/' . $guild . '/members?limit=' . $limit . $afterStr);

        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Authorization: Bot ' . $keys['argusDiscordOAuth2BotToken'],
        ]);
        self::$lastCurlHeaders = [];
        curl_setopt($curl, CURLOPT_HEADERFUNCTION, ['Argus\Discord\API', 'parseHeadersResponse']);

        $response     = json_decode(curl_exec($curl));
        $responseCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if (intval($responseCode) == 429) {
            sleep(self::getSleepTime($response));
            return self::requestMembersData($guild, $keys, $limit, $after);
        }

        self::$lastCurlHeaders = [];
        return $response;
    }


    protected static function requestSingleMemberData ($guild, $keys, $member) {
        return self::simpleAPIRequest('https://discordapp.com/api/guilds/' . $guild . '/members/' . $member, $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function requestMessagesFromChannel ($channel, $keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/channels/' . $channel . '/messages?limit=50', $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function requestPinnedMessagesFromChannel ($channel, $keys) {
        return self::simpleAPIRequest('https://discordapp.com/api/channels/' . $channel . '/pins', $keys['argusDiscordOAuth2BotToken']);
    }


    protected static function sendMessageRequest ($webhook, $keys, $message, $tts = 0, $file = false) {
        $curl = curl_init($webhook);

        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        $data = ['content' => $message, 'tts' => $tts];
        if ($file) {
            curl_setopt($curl, CURLOPT_POST, 1);
            $data['file'] = curl_file_create($file, mime_content_type($file), basename($file) . PHP_EOL);
        }
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Content-Type: multipart/form-data',
            'Accept: application/json',
            'Authorization: Bot ' . $keys['argusDiscordOAuth2BotToken'],
        ]);

        $response = json_decode(curl_exec($curl));
        curl_close($curl);
        return $response;
    }


    protected static function requestBearerToken ($code, $keys, $clientID, $clientSecret) {
        $curl = curl_init('https://discordapp.com/api/oauth2/token');

        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_POSTFIELDS, [
            'grant_type'    => 'authorization_code',
            'client_id'     => $clientID,
            'client_secret' => $clientSecret,
            'code'          => $code
        ]);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Authorization: Bearer ' . $keys['argusDiscordOAuth2AccessToken'],
        ]);

        $response = json_decode(curl_exec($curl));
        curl_close($curl);
        return $response;
    }


    protected static function requestBearerTokenRefresh ($clientID, $clientSecret, $refreshToken) {
        $curl = curl_init('https://discordapp.com/api/oauth2/token');

        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_POSTFIELDS, [
            'grant_type'    => 'refresh_token',
            'client_id'     => $clientID,
            'client_secret' => $clientSecret,
            'refresh_token' => $refreshToken
        ]);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json'
        ]);

        $response = json_decode(curl_exec($curl));
        curl_close($curl);
        return $response;
    }


    protected static function buildAvatarLink ($user, $size):string {
        if (!isset($user->id) || !isset($user->avatar) || !$user->avatar) return '';

        $requestSize = 16;
        while ($size > $requestSize) $requestSize = $requestSize * 2;

        $avatar = 'https://cdn.discordapp.com/avatars/' . $user->id . '/' . $user->avatar . '.png?size=' . $requestSize;

        if (!self::checkUrlAccess($avatar)) return '';
        return $avatar;
    }


    protected static function buildAppIconLink ($app, $size):string {
        $icon = 'https://cdn.discordapp.com/app-icons/' . $app->id . '/' . $app->icon . '.png?size=' . $size;
        if (!self::checkUrlAccess($icon)) return '';
        return $icon;
    }


    protected static function buildAuthorizationLink ($state):string {
        $query = http_build_query([
            'client_id'     => get_option('argusDiscordOAuth2ClientID'),
            'response_type' => 'code',
            'scope'         => 'identify email guilds',
            'state'         => $state
        ]);
        return 'https://discord.com/oauth2/authorize?' . $query;
    }


    protected static function buildShareLink ($state):string {
        $query = http_build_query([
            'client_id'     => get_option('argusDiscordOAuth2ClientID'),
            'response_type' => 'code',
            'scope'         => 'webhook.incoming',
            'state'         => $state . '-' . get_the_ID()
        ]);
        return 'https://discord.com/oauth2/authorize?' . $query;
    }


    protected static function buildDefaultDiscordWidget ($guild, $theme, $height):string {
        $query = http_build_query([
            'id'    => wp_strip_all_tags($guild),
            'theme' => $theme
        ]);
        return '<iframe src="https://discordapp.com/widget?' . $query . '" height="' . $height . '" width="100%" allowtransparency="true" frameborder="0"></iframe>';
    }


    private static function simpleAPIRequest ($url, $key, $type = 'Bot') {
        $curl = curl_init($url);

        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Authorization: ' . $type . ' ' . $key,
        ]);

        $response = json_decode(curl_exec($curl));
        curl_close($curl);
        return $response;
    }


    private static function checkUrlAccess ($link):bool {
        $curl = curl_init($link);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        curl_exec($curl);
        $response = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        return $response == 200;
    }


    private static function getSleepTime ($response) {
        $intervals = [];
        if (isset(self::$lastCurlHeaders['x-ratelimit-reset-after'])) {
            $intervals[] = self::$lastCurlHeaders['x-ratelimit-reset-after'];
        }
        if (isset($response->retry_after)) {
            $intervals[] = $response->retry_after;
        }

        $sleep = min($intervals);
        if ($sleep > self::$sleepRequestTolerance) $sleep = self::$forceRequestRetryTime;

        return $sleep;
    }


    static function parseHeadersResponse ($curl, $header):int {
        $parsedString = explode(':', $header, 2);
        if (isset($parsedString[0]) && isset($parsedString[1]) && trim($parsedString[0]) && trim($parsedString[1])) {
            self::$lastCurlHeaders[ trim($parsedString[0]) ] = trim($parsedString[1]);
        }
        return strlen($header);
    }

}
