<?php

namespace Argus\Discord;

use Argus\Log;

class APIErrorHandler extends API {


    protected static function processSendMessageRequest ($hook, $keys, $message, $tts, $file) {
        $response = self::sendMessageRequest($hook, $keys, $message, $tts, $file);

        if (isset($response->message)) {
            $serverResponseMessage = __("During posting message to the channel, Discord responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if ((isset($response->code) && $response->code == 50027) || self::checkForAccessErrors($response)) {
                $response->message = self::getRevokedAccessMessage($serverResponseMessage);
                update_option('argusDiscordErrorMessage', $response->message);
                Log::addMessage($response->message, 1);

            } else {
                update_option('argusDiscordErrorMessage', $serverResponseMessage);
                Log::addMessage($serverResponseMessage, 1, 20);
                $response->message = $serverResponseMessage;
            }

        }

        return $response;
    }


    protected static function processRequestToAddUserRole ($keys, $guild, $discordUserID, $discordRoleID, $wpUserID):bool {
        $response = self::requestToAddUserRole($keys, $guild, $discordUserID, $discordRoleID);

        if (isset($response->message)) {
            $wpUserName      = self::getWPUserNameTemplate($wpUserID);
            $discordUserName = self::getDiscordUserNameTemplate($discordUserID);
            $discordRoleName = self::getRoleNameTemplate($discordRoleID);

            $serverResponseMessage = sprintf(__("During adding Discord role %s to Discord profile %s while syncing WP user %s, Discord responded with a message:", 'argus-wordpress-plugin'), $discordRoleName, $discordUserName, $wpUserName) . self::getDiscordAPIErrorTemplate($response);

            if (isset($response->code) && $response->code == 10011) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly %s role does not exist on the Discord server. Try to update plugin data.", 'argus-wordpress-plugin'), $discordRoleName);
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 10);

            } elseif (isset($response->code) && $response->code == 50013) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly %s role is higher than the plugin role. The plugin can assign only lower roles, click %s here %s to know more.", 'argus-wordpress-plugin'), $discordRoleName, "<a href='#' onclick='argusOpenHelpTabSection(\"tab-link-argus-help-tab-linked-error-roles-hierarchy\");'>", '</a>');
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 20);

            } elseif (isset($response->code) && $response->code == 10013) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly user %s has an incorrect Discord profile connected. You can open the user's profile by clicking on the name.", 'argus-wordpress-plugin'), $wpUserName);
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 10);

            } elseif ((isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return true;
    }


    protected static function processRequestToRemoveUserRole ($keys, $guild, $discordUserID, $discordRoleID, $wpUserID):bool {
        $response = self::requestToRemoveUserRole($keys, $guild, $discordUserID, $discordRoleID);

        if (isset($response->message)) {
            $wpUserName      = self::getWPUserNameTemplate($wpUserID);
            $discordUserName = self::getDiscordUserNameTemplate($discordUserID);
            $discordRoleName = self::getRoleNameTemplate($discordRoleID);

            $serverResponseMessage = sprintf(__("During removing Discord role %s from Discord profile %s while syncing WP user %s, Discord responded with a message:", 'argus-wordpress-plugin'), $discordRoleName, $discordUserName, $wpUserName) . self::getDiscordAPIErrorTemplate($response);

            if (isset($response->code) && $response->code == 10011) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly %s role does not exist on the Discord server. Try to update plugin data.", 'argus-wordpress-plugin'), $discordRoleName);
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 10);

            } elseif (isset($response->code) && $response->code == 50013) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly %s role is higher than the plugin role. The plugin can assign only lower roles, click %s here %s to know more.", 'argus-wordpress-plugin'), $discordRoleName, "<a href='#' onclick='argusOpenHelpTabSection(\"tab-link-argus-help-tab-linked-error-roles-hierarchy\");'>", '</a>');
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 20);

            } elseif (isset($response->code) && $response->code == 10013) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly user %s has an incorrect Discord profile connected. You can open the user's profile by clicking on the name.", 'argus-wordpress-plugin'), $wpUserName);
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 10);

            } elseif ((isset($response->code) && $response->code == 10013) || (isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return true;
    }


    protected static function processRequestToRemoveUserFromServer ($keys, $guild, $discordUserID, $wpUserID):bool {
        $response = self::requestToRemoveUserFromServer($keys, $guild, $discordUserID);

        if (isset($response->message)) {
            $wpUserName      = self::getRoleNameTemplate($wpUserID);
            $discordUserName = self::getDiscordUserNameTemplate($discordUserID);

            $serverResponseMessage = sprintf(__("During removing Discord user %s with ID %s from Discord server while disconnecting Discord profile from  WP user %s, Discord responded with a message:", 'argus-wordpress-plugin'), $discordUserName, $discordUserID, $wpUserName) . self::getDiscordAPIErrorTemplate($response);

            if (isset($response->code) && $response->code == 10013) {
                $notificationMessage = $serverResponseMessage . __("Possibly user already left the Discord server.", 'argus-wordpress-plugin');
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 10);

            } elseif ((isset($response->code) && $response->code == 10013) || (isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return true;
    }


    protected static function processRequestSingleMemberData ($guild, $keys, $discordUserID, $suppressUnknownUserError = false) {
        $response = self::requestSingleMemberData($guild, $keys, $discordUserID);

        if (isset($response->message)) {
            if ($suppressUnknownUserError && isset($response->code) && $response->code == 10013) return 'suppressed';

            $discordUserName = self::getDiscordUserNameTemplate($discordUserID);

            $serverResponseMessage = sprintf(__("During data request of the guild member %s, Discord responded with a message:", 'argus-wordpress-plugin'), $discordUserName) . self::getDiscordAPIErrorTemplate($response);

            if (isset($response->code) && $response->code == 10013) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly member %s has an incorrect Discord profile connected. You can open the user's profile by clicking on the name.", 'argus-wordpress-plugin'), $discordUserName);
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 10);

            } elseif ((isset($response->code) && $response->code == 10013) || (isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestAuthorizedProfileData ($keys) {
        $response = self::requestAuthorizedProfileData($keys);

        if (isset($response->message)) {

            $serverResponseMessage = __("During authorized API profile data synchronization Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if (self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage, 40);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestAuthorizedAppData ($keys) {
        $response = self::requestAuthorizedAppData($keys);

        if (isset($response->message)) {

            $serverResponseMessage = __("During authorized API application data synchronization Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if (self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage, 40);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestAllRoles ($guild, $keys) {
        $response = self::requestAllRoles($guild, $keys);

        if (isset($response->message)) {

            $serverResponseMessage = __("During request of all guild roles data Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if ((isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestAllChannels ($guild, $keys) {
        $response = self::requestAllChannels($guild, $keys);

        if (isset($response->message)) {

            $serverResponseMessage = __("During request of all guild channels list Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if ((isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestChannelInvites ($channel, $keys) {
        $response = self::requestChannelInvites($channel, $keys);

        if (isset($response->message)) {
            $discordChannelName = self::getChannelNameTemplate($channel);

            $serverResponseMessage = sprintf(__("During invite link data synchronization of the channel %s, Discord responded with a message:", 'argus-wordpress-plugin'), $discordChannelName) . self::getDiscordAPIErrorTemplate($response);

            if (self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        if (!isset($response->code)) {
            $notificationMessage = sprintf(__("During invite link data synchronization of the channel %s Discord limited amount of links. Please fetch links data again after %s seconds", 'argus-wordpress-plugin'), self::getChannelNameTemplate($channel), $response->retry_after);
            update_option('argusDiscordErrorMessage', $notificationMessage);
            Log::addMessage($notificationMessage, 1, 20);
            return false;
        }

        return $response;
    }


    protected static function processRequestGuildData ($guild, $keys) {
        $response = self::requestGuildData($guild, $keys);

        if (isset($response->message)) {

            $serverResponseMessage = __("During guild data synchronization Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if ((isset($response->code) && $response->code == 10004) || self::checkForAccessErrors($response)) {
                self::saveRevokedAccessMessage($serverResponseMessage, 40);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestMembersData ($guild, $keys, $step, $after) {
        $response = self::requestMembersData($guild, $keys, $step, $after);

        if (isset($response->message)) {

            $serverResponseMessage = __("During all guild members data synchronization Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response);

            if ($response->code == 50001) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly <code>SERVER MEMBERS INTENT</code> bot option is not activated. Click %s here %s to know more.", 'argus-wordpress-plugin'), "<a href='#' onclick='argusOpenHelpTabSection(\"tab-link-argus-help-tab-linked-error-members-intent\");'>", '</a>');
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1, 20);

            } elseif ($response->code == 10004 || substr($response->message, 0, 3) == '401' || substr($response->message, 0, 3) == '403') {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestPinnedMessagesFromChannel ($channel, $keys) {
        $response = self::requestPinnedMessagesFromChannel($channel, $keys);

        if (isset($response->message)) {
            $discordChannelName = self::getChannelNameTemplate($channel);

            $serverResponseMessage = sprintf(__("During pinned messages synchronization from channel %s Discord server responded with a message:", 'argus-wordpress-plugin'), $discordChannelName) . self::getDiscordAPIErrorTemplate($response);

            if ($response->code == 50001) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly channel %s is private. You need to add website bot to this channel for plugin to be able to see this channel. Click %s here %s to know more.", 'argus-wordpress-plugin'), $discordChannelName, "<a href='#' onclick='argusOpenHelpTabSection(\"tab-link-argus-help-tab-linked-error-private-channels\");'>", '</a>');
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1);

            } elseif ($response->code == 10004 || substr($response->message, 0, 3) == '401' || substr($response->message, 0, 3) == '403') {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestMessagesFromChannel ($channel, $keys) {
        $response = self::requestMessagesFromChannel($channel, $keys);

        if (isset($response->message)) {
            $discordChannelName = self::getChannelNameTemplate($channel);

            $serverResponseMessage = sprintf(__("During messages synchronization from channel %s Discord server responded with a message:", 'argus-wordpress-plugin'), $discordChannelName) . self::getDiscordAPIErrorTemplate($response);

            if ($response->code == 50001) {
                $notificationMessage = $serverResponseMessage . sprintf(__("Possibly channel %s is private. You need to add website bot to this channel for plugin to be able to see this channel. Click %s here %s to know more.", 'argus-wordpress-plugin'), $discordChannelName, "<a href='#' onclick='argusOpenHelpTabSection(\"tab-link-argus-help-tab-linked-error-private-channels\");'>", '</a>');
                update_option('argusDiscordErrorMessage', $notificationMessage);
                Log::addMessage($notificationMessage, 1);

            } elseif ($response->code == 10004 || substr($response->message, 0, 3) == '401' || substr($response->message, 0, 3) == '403') {
                self::saveRevokedAccessMessage($serverResponseMessage);

            } else {
                self::saveMessage($serverResponseMessage);
            }

            return false;
        }

        return $response;
    }


    protected static function processRequestBearerToken ($code, $keys, $clientID, $clientSecret) {
        $response = self::requestBearerToken($code, $keys, $clientID, $clientSecret);

        if (isset($response->error) || isset($response->message)) {

            if (isset($response->message)) {
                update_option('argusDiscordErrorMessage', __("Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response));

            } elseif ($response->error == 'invalid_client') {
                update_option('argusDiscordErrorMessage', __("During the last authorization attempt, wrong Client Secret key was passed.", 'argus-wordpress-plugin'));

            } else {
                update_option('argusDiscordErrorMessage', __("Discord server responded with a message:", 'argus-wordpress-plugin') . '&nbsp;&nbsp;<code>' . esc_html($response->error) . '</code>');
            }

            return false;
        }

        return $response;
    }


    protected static function tokenCheck ($keys):bool {
        $response = self::requestAuthorizedAppData($keys);

        if (isset($response->message)) {
            update_option('argusDiscordErrorMessage', __("Bot token might be incorrect. Discord server responded with a message:", 'argus-wordpress-plugin') . self::getDiscordAPIErrorTemplate($response));
            return false;
        }

        return true;
    }


    protected static function processRequestBearerTokenRefresh ($clientID, $clientSecret, $refreshToken) {
        $response = self::requestBearerTokenRefresh($clientID, $clientSecret, $refreshToken);

        if (isset($response->error) || isset($response->message)) {
            update_option('argusDiscordErrorMessage', __("Discord server did not refreshed access token. Server responded with a message:", 'argus-wordpress-plugin') . '&nbsp;&nbsp;<code>' . (isset($response->message) ? esc_html($response->message) : '') . (isset($response->error) ? esc_html($response->error) : '') . '</code>');
            return false;
        }

        return $response;
    }


    // Check for errors 50001, 401 and 403
    private static function checkForAccessErrors ($response):bool {
        if ((isset($response->code) && $response->code == 50001) || substr($response->message, 0, 3) == '401' || substr($response->message, 0, 3) == '403') {
            return true;
        }
        return false;
    }


    private static function getRevokedAccessMessage ($prefixMessage = ''):string {
        return $prefixMessage . __("Possibly access to the Discord server was revoked.", 'argus-wordpress-plugin');
    }


    private static function saveRevokedAccessMessage ($message, $level = 30):void {
        $message = self::getRevokedAccessMessage($message);
        update_option('argusDiscordErrorMessage', $message);
        Log::addMessage($message, 1, $level);
    }


    private static function saveMessage ($message, $level = 30):void {
        update_option('argusDiscordErrorMessage', $message);
        Log::addMessage($message, 1, $level);
    }


    private static function getDiscordAPIErrorTemplate ($response):string {
        return '&nbsp;&nbsp;<code>' . esc_html($response->message) . '</code>&nbsp;&nbsp;';
    }


    private static function getRoleNameTemplate ($id):string {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_roles` WHERE `role_id`='%d' LIMIT 1;", $id));
        if (!$data) return '';
        return '<i>' . unserialize($data->object)->name . '</i>';
    }


    private static function getChannelNameTemplate ($id):string {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_channels` WHERE `channel_id`='%d' LIMIT 1;", $id));
        if (!$data) return '';
        return '<i>' . unserialize($data->object)->name . '</i>';
    }


    private static function getDiscordUserNameTemplate ($id):string {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_members` WHERE `user_id`='%d' LIMIT 1;", $id));
        if ($data) {
            $data = unserialize($data->object);
            if (!isset($data->user->username) || !isset($data->user->discriminator)) {
                return '';
            }
        } else return '';
        return '<i>' . $data->user->username . '#' . $data->user->discriminator . '</i>';
    }


    private static function getWPUserNameTemplate ($id):string {
        $user = get_user_by('id', $id);
        $link = get_edit_user_link($id);
        if (!isset($user->data->user_nicename)) return '';
        return '<a href="' . $link . '" target="_blank"><i>' . $user->data->user_nicename . '</i></a>';
    }


    protected static function saveMySQLError ($prefix) {
        global $wpdb;
        update_option('argusDiscordErrorMessage', $prefix . ' ' . sprintf(__("database responded with an error %s on query %s", 'argus-wordpress-plugin'), '&nbsp;&nbsp;<code>' . $wpdb->last_error . '</code>&nbsp;&nbsp;', '&nbsp;&nbsp;<code>' . $wpdb->last_query . '</code>&nbsp;&nbsp;') . ' ' . __("If you updated to a new major version of the plugin on top of the previous one check update documentation. Maybe you need to completely reinstall the plugin.", 'argus-wordpress-plugin'));
    }

}
