<?php

namespace Argus\Discord;

use stdClass;

class Data extends APIErrorHandler {


    static function getLastError () {
        $error = get_option('argusDiscordErrorMessage');
        update_option('argusDiscordErrorMessage', '');
        return $error;
    }


    static function sendMessage ($message, $webhook = false, $tts = 0, $file = false) {
        if (!$webhook) $webhook = get_option('argusDiscordWebhook');

        return self::processSendMessageRequest($webhook, self::getAccessKeys(), $message, $tts, $file ? get_attached_file($file) : false);
    }


    static function getAuthorizationLink ($state = 'argus-discord-user-authorization'):string {
        return self::buildAuthorizationLink($state);
    }


    static function getShareLink ($state = 'argus-discord-share-page'):string {
        return self::buildShareLink($state);
    }


    static function getAuthorizedProfileData ($keys) {
        return self::requestAuthorizedProfileData($keys);
    }


    static function getAvatarLinkOfTheGuildMember ($user, $size):string {
        $avatar = self::buildAvatarLink($user, $size);
        if ($avatar) return $avatar;
        return Data::getDiscordIconURL('accent');
    }


    static function getAvatarLinkFromAuthorizedUsers ($userID, $size, $thumbnail = true):string {
        $user = self::findUserInAuthorizationData($userID);
        if (!$user || !isset($user->id)) {
            if (!$thumbnail) return '';
            return Data::getDiscordIconURL('accent');
        }

        $avatar = self::buildAvatarLink($user, $size);

        if ($avatar) return $avatar;
        if (!$thumbnail) return '';
        return Data::getDiscordIconURL('accent');
    }


    static function getDefaultDiscordWidget ($guild, $theme, $height):string {
        return self::buildDefaultDiscordWidget($guild, $theme, $height);
    }


    static function getAppIconLink ($user, $size):string {
        $icon = self::buildAppIconLink($user, $size);
        if ($icon) return $icon;
        return Data::getDiscordIconURL('accent');
    }


    static function getWebhookChannelName () {
        $argusDiscordOAuth2TokenResponse = unserialize(get_option('argusDiscordOAuth2TokenResponse'));
        if (!isset($argusDiscordOAuth2TokenResponse->webhook) || !isset($argusDiscordOAuth2TokenResponse->webhook->channel_id)) {
            return '[' . __("warning, no access to the webhook channel", 'argus-wordpress-plugin') . ']';
        }
        $channelID = $argusDiscordOAuth2TokenResponse->webhook->channel_id;
        return self::getChannelName($channelID);
    }


    static function getSingleMemberData ($discordUserID, $suppressUnknownUserError = false) {
        self::fetchSingleMemberData($discordUserID, $suppressUnknownUserError);

        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_members` WHERE `user_id`='%d' LIMIT 1;", $discordUserID));
        if (!isset($data->object)) return new stdClass();
        return unserialize($data->object);
    }


    static function checkIfUserInGuild ($discordUserID):bool {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT COUNT(*) AS `count` FROM `{$wpdb->prefix}argus_discord_members` WHERE `user_id`='%d' LIMIT 1;", $discordUserID));
        if ($data->count) return true;
        return false;
    }


    static function getRoleName ($id):string {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_roles` WHERE `role_id`='%d' LIMIT 1;", $id));
        if (!$data) return '';
        return unserialize($data->object)->name;
    }


    static function getAllRolesList ():array {
        $list = [];
        global $wpdb;

        $data = $wpdb->get_results("SELECT `object` FROM `{$wpdb->prefix}argus_discord_roles`;");
        if ($data) {
            foreach ($data as $role) {
                $role   = unserialize($role->object);
                $list[] = (object)[
                    'id'   => $role->id,
                    'name' => $role->name,
                ];
            }
        }

        return $list;
    }


    static function getAllRoleBindingsList () {
        global $wpdb;
        $bindings = $wpdb->get_results("SELECT `wp_role`, `discord_role`, `direction` FROM `{$wpdb->prefix}argus_discord_bound_roles`;");
        $roles    = self::getAllRolesList();
        foreach ($bindings as $key => $binding) {
            foreach ($roles as $role) {
                if ($role->id == $binding->discord_role) {
                    $binding->discord_role_name = $role->name;
                    break;
                }
            }
        }
        return $bindings;
    }


    static function getChannelName ($id):string {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_channels` WHERE `channel_id`='%d' LIMIT 1;", $id));
        if (!$data) return '[' . __("warning, no access to the channel", 'argus-wordpress-plugin') . ' ' . $id . ']';
        return unserialize($data->object)->name;
    }


    static function getAllChannelsList () {
        $list = [];
        global $wpdb;

        $data = $wpdb->get_results("SELECT `object` FROM `{$wpdb->prefix}argus_discord_channels`;");
        if ($data) {
            foreach ($data as $channel) {
                $channel = unserialize($channel->object);
                $list[]  = (object)[
                    'id'   => $channel->id,
                    'name' => $channel->name,
                ];
            }
        }

        return $list;
    }


    static function getAllChannelsData () {
        global $wpdb;
        $results = $wpdb->get_results("SELECT `channel_id` AS `id`, `object` FROM `{$wpdb->prefix}argus_discord_channels`;");
        foreach ($results as $channel) $channel->object = unserialize($channel->object);
        return $results;
    }


    static function getAllInvitesData ():array {
        global $wpdb;
        $results = $wpdb->get_results("SELECT `invite_object` FROM `{$wpdb->prefix}argus_discord_channels`;");
        $invites = [];
        foreach ($results as $invite) {
            $unserialized = unserialize($invite->invite_object);
            if ($unserialized) $invites[] = $unserialized;
        }
        return $invites;
    }


    static function getInviteLink ($channel = null) {
        if (!$channel) return '';
        global $wpdb;

        $query  = $wpdb->prepare("SELECT `invite_object` FROM `{$wpdb->prefix}argus_discord_channels` WHERE `channel_id` = '%d' LIMIT 1;", $channel);
        $result = $wpdb->get_row($query);

        if (!isset($result->invite_object) || !unserialize($result->invite_object)) {
            $response = self::processRequestChannelInvites($channel, self::getAccessKeys());
            if ($response === false || !self::insertChanelInvite($channel, $response)) return '';

            $result = $wpdb->get_row($query);
            if (!isset($result->invite_object) || !unserialize($result->invite_object)) return '';
        }

        $object = unserialize($result->invite_object);
        if (!isset($object->link)) return '';

        return $object->link;
    }


    static function getAllMessagesData ($channel = null, $quantity = 20, $onlyPinned = false):array {
        if (!$channel) return [];
        global $wpdb;
        $pinned = '';
        if ($onlyPinned) $pinned = ' AND `pinned` = 1 ';
        $results  = $wpdb->get_results($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_messages` WHERE `channel_id` = '%d' " . $pinned . ' ORDER BY `timestamp` DESC LIMIT %d;', $channel, $quantity));
        $messages = [];
        foreach ($results as $message) $messages[] = unserialize($message->object);
        return $messages;
    }


    static function getDiscordIconURL ($style = 'white'):string {
        return '/wp-content/plugins/argus-wordpress-plugin/components/discord/views/styles/ui-kit/images/icon-' . $style . '.svg';
    }


    static function findProfile ($user) {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_members` WHERE `user_id`='%d' LIMIT 1;", $user));
        if (!$data) {
            return (object)[
                'id'            => $user,
                'username'      => '',
                'discriminator' => '',
            ];
        } else {
            $data = unserialize($data->object);
            return (object)[
                'id'            => $user,
                'username'      => $data->user->username,
                'discriminator' => $data->user->discriminator,
            ];
        }
    }


    static function findUserInAuthorizationData ($user) {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT `object` FROM `{$wpdb->prefix}argus_discord_collected_data` WHERE `user_id`='%d' LIMIT 1;", $user));
        if (!$data) {
            return (object)[
                'id'            => $user,
                'username'      => '',
                'discriminator' => '',
                'avatar'        => '',
            ];
        }
        return unserialize($data->object);
    }


    static function saveCollectedUserData ($user, $data) {
        global $wpdb;

        $result = $wpdb->replace($wpdb->prefix . 'argus_discord_collected_data', [
            'user_id' => $user,
            'object'  => serialize($data),
        ], ['%d', '%s']);

        if ($result) return true;
        return false;
    }


    static function addUserRole ($discordUserID, $discordRoleID, $wpUserID):bool {
        return self::processRequestToAddUserRole(self::getAccessKeys(), get_option('argusDiscordGuild'), $discordUserID, $discordRoleID, $wpUserID);
    }


    static function addRoleBinding ($wpRole, $discordRole, $direction) {
        global $wpdb;
        return $wpdb->replace($wpdb->prefix . 'argus_discord_bound_roles', [
            'wp_role'      => $wpRole,
            'direction'    => $direction,
            'discord_role' => $discordRole,
        ], ['%s', '%d', '%d']);
    }


    static function removeUserRole ($discordUserID, $discordRoleID, $wpUserID):bool {
        return self::processRequestToRemoveUserRole(self::getAccessKeys(), get_option('argusDiscordGuild'), $discordUserID, $discordRoleID, $wpUserID);
    }


    static function removeUserFromServer ($discordUserID, $wpUserID):bool {
        return self::processRequestToRemoveUserFromServer(self::getAccessKeys(), get_option('argusDiscordGuild'), $discordUserID, $wpUserID);
    }


    static function deleteRoleBinding ($direction, $wpRole, $discordRole) {
        global $wpdb;
        return $wpdb->delete($wpdb->prefix . 'argus_discord_bound_roles', ['direction' => intval($direction), 'wp_role' => wp_strip_all_tags($wpRole), 'discord_role' => intval($discordRole)]);
    }


    static function fetchAuthorizedProfile () {
        $response = self::processRequestAuthorizedProfileData(self::getAccessKeys());
        if ($response === false) return false;
        update_option('argusDiscordDataProfile', serialize($response));
        return true;
    }


    static function fetchAuthorizedApp ():bool {
        $response = self::processRequestAuthorizedAppData(self::getAccessKeys());
        if ($response === false) return false;
        update_option('argusDiscordDataApp', serialize($response));
        return true;
    }


    static function fetchAllRoles ():bool {
        $response = self::processRequestAllRoles(get_option('argusDiscordGuild'), self::getAccessKeys());
        if ($response === false) return false;

        global $wpdb;
        $wpdb->query('START TRANSACTION;');
        self::prepareToLargeInsert('argus_discord_roles');

        foreach ($response as $role) {
            $wpdb->hide_errors();
            $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_roles', [
                'role_id' => $role->id,
                'object'  => serialize($role),
            ], ['%d', '%s']);
            $wpdb->show_errors();
            if ($saveStatus === false) {
                self::saveMySQLError(__("During saving of all guild roles data", 'argus-wordpress-plugin'));
                $wpdb->query('ROLLBACK;');
                return false;
            }
        }

        self::restoreAfterLargeInsert();
        $wpdb->query('COMMIT;');
        return true;
    }


    static function fetchSingleChannel ($channelID):bool {
        global $wpdb;

        $wpdb->query('START TRANSACTION;');

        $wpdb->delete($wpdb->prefix . 'argus_discord_messages', ['channel_id' => $channelID]);

        $response = self::processRequestMessagesFromChannel($channelID, self::getAccessKeys());
        if ($response === false || !self::insertChannelMessages($response)) {
            $wpdb->query('ROLLBACK;');
            return false;
        }

        $response = self::processRequestPinnedMessagesFromChannel($channelID, self::getAccessKeys());
        if ($response === false || !self::insertChannelMessages($response, true)) {
            $wpdb->query('ROLLBACK;');
            return false;
        }

        $wpdb->query('COMMIT;');
        return true;
    }


    static function fetchAllChannels () {
        $response = self::processRequestAllChannels(get_option('argusDiscordGuild'), self::getAccessKeys());
        if ($response === false) return false;

        foreach ($response as $key => $channel) {
            if ($channel->type != 0 && $channel->type != 2) unset($response[ $key ]);
        }

        $response = array_values($response);
        if (count($response)) {

            global $wpdb;
            $wpdb->query('START TRANSACTION;');
            self::prepareToLargeInsert('argus_discord_channels');

            foreach ($response as $channel) {
                $wpdb->hide_errors();
                $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_channels', [
                    'channel_id' => $channel->id,
                    'object'     => serialize($channel),
                ], ['%d', '%s']);
                $wpdb->show_errors();
                if ($saveStatus === false) {
                    self::saveMySQLError(__("During saving of all guild channels list", 'argus-wordpress-plugin'));
                    $wpdb->query('ROLLBACK;');
                    return false;
                }
            }

            self::restoreAfterLargeInsert();
            $wpdb->query('COMMIT;');
            return true;

        }
    }


    static function fetchAllChannelsInviteLinks ():bool {
        $errors = false;

        $invites = self::getAllInvitesData();
        if (count($invites)) {
            foreach ($invites as $invite) {

                $response = self::processRequestChannelInvites($invite->channel->id, self::getAccessKeys());
                if ($response === false) {
                    $errors = true;
                    continue;
                }

                if (!self::insertChanelInvite($invite->channel->id, $response)) return false;
            }
        }

        if ($errors) return false;
        return true;
    }


    static function fetchGuild ():bool {
        $response = self::processRequestGuildData(get_option('argusDiscordGuild'), self::getAccessKeys());
        if ($response === false) return false;
        unset($response->roles);
        update_option('argusDiscordDataGuild', serialize($response));
        return true;
    }


    static function fetchSingleMemberData ($discordUserID, $suppressUnknownUserError = false) {
        $response = self::processRequestSingleMemberData(get_option('argusDiscordGuild'), self::getAccessKeys(), $discordUserID, $suppressUnknownUserError);
        if ($suppressUnknownUserError && $response == 'suppressed') return true;
        if (!$response || !isset($response->user->id)) return false;

        global $wpdb;
        $wpdb->query('START TRANSACTION;');
        $wpdb->hide_errors();
        $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_members', [
            'user_id' => $response->user->id,
            'object'  => serialize($response),
        ], ['%d', '%s']);
        $wpdb->show_errors();

        if ($saveStatus !== false) {
            foreach ($response->roles as $role) {
                $wpdb->hide_errors();
                $wpdb->delete($wpdb->prefix . 'argus_discord_assigned_roles', ['user_id' => wp_strip_all_tags($response->user->id)]);
                $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_assigned_roles', [
                    'user_id' => $response->user->id,
                    'role_id' => $role,
                ], ['%d', '%d']);
                $wpdb->show_errors();
            }
        }

        if ($saveStatus === false) {
            self::saveMySQLError(__("During saving guild members data", 'argus-wordpress-plugin'));
            $wpdb->query('ROLLBACK;');
            return false;
        }

        $wpdb->query('COMMIT;');
        return true;
    }


    static function fetchMembers ($bot = false, $step = 1000):bool {
        global $wpdb;

        $wpdb->query('START TRANSACTION;');
        self::prepareToLargeInsert('argus_discord_members', false);
        self::prepareToLargeInsert('argus_discord_assigned_roles', false);

        if (!self::fetchMembersRecursion($bot, $step)) {
            $wpdb->query('ROLLBACK;');
            return false;
        }

        self::restoreAfterLargeInsert();
        self::restoreAfterLargeInsert();
        $wpdb->query('COMMIT;');
        return true;
    }


    private static function fetchMembersRecursion ($bot, $step, $after = 0):bool {
        $response = self::processRequestMembersData(get_option('argusDiscordGuild'), self::getAccessKeys(), $step, $after);
        if (is_array($response) && !count($response)) return true;
        if (!$response) return false;

        global $wpdb;
        foreach ($response as $member) {
            if ($bot || !(isset($member->user->bot) && $member->user->bot)) {

                $wpdb->hide_errors();
                $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_members', [
                    'user_id' => $member->user->id,
                    'object'  => serialize($member),
                ], ['%d', '%s']);
                $wpdb->show_errors();

                if ($saveStatus !== false) {
                    foreach ($member->roles as $role) {
                        $wpdb->hide_errors();
                        $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_assigned_roles', [
                            'user_id' => $member->user->id,
                            'role_id' => $role,
                        ], ['%d', '%d']);
                        $wpdb->show_errors();
                    }
                }

                if ($saveStatus === false) {
                    self::saveMySQLError(__("During saving guild members data", 'argus-wordpress-plugin'));
                    $wpdb->query('ROLLBACK;');
                    return false;
                }

            }
        }

        if (count($response) == $step) {
            return self::fetchMembersRecursion($bot, $step, end($response)->user->id);
        }
        return true;
    }


    static function fetchMessagesFromAllChannels ():bool {
        $errors = false;

        $channels = self::getAllChannelsData();
        if (count($channels)) {
            global $wpdb;

            $wpdb->query('START TRANSACTION;');
            $wpdb->delete($wpdb->prefix . 'argus_discord_messages', ['pinned' => 0]);
            self::prepareToLargeInsert('argus_discord_messages', true, false);

            foreach ($channels as $channel) {

                $response = self::processRequestMessagesFromChannel($channel->id, self::getAccessKeys());
                if ($response === false) {
                    $errors = true;
                    continue;
                }

                if (!self::insertChannelMessages($response)) return false;
            }

            self::restoreAfterLargeInsert();
            $wpdb->query('COMMIT;');
        }

        if ($errors) return false;
        return true;
    }


    static function fetchPinnedMessagesFromAllChannels ():bool {
        $errors = false;

        $channels = self::getAllChannelsData();
        if (count($channels)) {
            global $wpdb;

            $wpdb->query('START TRANSACTION;');
            $wpdb->delete($wpdb->prefix . 'argus_discord_messages', ['pinned' => 1]);

            foreach ($channels as $channel) {

                $response = self::processRequestPinnedMessagesFromChannel($channel->id, self::getAccessKeys());
                if ($response === false) {
                    $errors = true;
                    continue;
                }

                if (!self::insertChannelMessages($response, true)) return false;
            }

            $wpdb->query('COMMIT;');
        }

        if ($errors) return false;
        return true;
    }


    private static function insertChanelInvite ($channelID, $response):bool {
        global $wpdb;

        $response->link = 'https://discord.gg/' . $response->code;
        $wpdb->hide_errors();
        $saveStatus = $wpdb->update($wpdb->prefix . 'argus_discord_channels', ['invite_object' => serialize($response)], ['channel_id' => $channelID], ['%s'], ['%d']);
        $wpdb->show_errors();
        if ($saveStatus === false) {
            self::saveMySQLError(__("During saving invite link", 'argus-wordpress-plugin'));
            return false;
        }

        return true;
    }


    private static function insertChannelMessages ($messages, $pinned = false):bool {
        global $wpdb;

        foreach ($messages as $message) {
            if ((!$pinned && !$message->pinned) || ($pinned && $message->pinned)) {

                $tenor = 0;
                if (isset($message->embeds[0]->provider->url) && strpos($message->embeds[0]->provider->url, 'tenor') !== false) {
                    $tenor = 1;
                }

                if (!isset($message->reactions)) $message->reactions = [];

                $wpdb->hide_errors();
                $saveStatus = $wpdb->replace($wpdb->prefix . 'argus_discord_messages', [
                    'id'         => $message->id,
                    'channel_id' => $message->channel_id,
                    'pinned'     => $message->pinned ? 1 : 0,
                    'timestamp'  => $message->timestamp,
                    'object'     => serialize($message),
                ], ['%d', '%d', '%d', '%s', '%s']);
                $wpdb->show_errors();
                if ($saveStatus === false) {
                    self::saveMySQLError(__("During saving messages data", 'argus-wordpress-plugin'));
                    $wpdb->query('ROLLBACK;');
                    return false;
                }

            }
        }

        return true;
    }


    static function authorizationLogout ($redirect = true) {
        global $wpdb;
        $options = [
            'argusDiscordOAuth2ClientID',
            'argusDiscordOAuth2ClientSecret',
            'argusDiscordOAuth2BotToken',
            'argusDiscordOAuth2TokenResponse',
            'argusDiscordOAuth2AccessToken',
            'argusDiscordWebhook',
            'argusDiscordGuild'
        ];

        foreach ($options as $option) update_option($option, '');

        $wpdb->query('TRUNCATE TABLE `' . $wpdb->prefix . 'argus_discord_bound_roles`;');

        Cron::unschedule();

        if ($redirect) {
            add_action('wp_loaded', function() {
                wp_redirect(admin_url('options-general.php?page=argus-discord-plugin-settings'));
            });
        }
    }


    static function getBearerToken ($code) {
        return self::requestBearerToken($code, self::getAccessKeys(), get_option('argusDiscordOAuth2ClientID'), get_option('argusDiscordOAuth2ClientSecret'));
    }


    static function fetchBearerTokenAndCheckBotToken ($code, $clientID, $clientSecret) {
        $response = self::processRequestBearerToken($code, self::getAccessKeys(), $clientID, $clientSecret);
        if ($response === false) {
            self::authorizationLogout(false);
            add_action('wp_loaded', function() {
                wp_redirect(admin_url('options-general.php?page=argus-discord-plugin-settings'));
            });
            return false;
        }

        // Cleaning and saving response token
        unset($response->guild->roles);
        update_option('argusDiscordOAuth2AccessToken', $response->access_token);
        update_option('argusDiscordWebhook', $response->webhook->url);
        update_option('argusDiscordOAuth2TokenResponse', serialize($response));
        update_option('argusDiscordGuild', $response->guild->id);

        // Checking bot token
        if (!self::tokenCheck(self::getAccessKeys())) {
            self::authorizationLogout();
            add_action('wp_loaded', function() {
                wp_redirect(admin_url('options-general.php?page=argus-discord-plugin-settings'));
            });
            return false;
        }

        // Downloading Discord app and guild data
        self::fetchAuthorizedApp();
        self::fetchGuild();

        Cron::schedule();
        setcookie('argusInitUpdate', 1);
        header('Location: ' . admin_url('options-general.php?page=argus-discord-plugin-settings'));
        exit;
    }


    static function refreshBearerToken ():bool {
        $dbTokenResponse = unserialize(get_option('argusDiscordOAuth2TokenResponse'));
        $response        = self::processRequestBearerTokenRefresh(get_option('argusDiscordOAuth2ClientID'), get_option('argusDiscordOAuth2ClientSecret'), $dbTokenResponse->refresh_token);
        if ($response === false) {
            update_option('argusDiscordOAuth2State', '');
            return false;
        }

        $dbTokenResponse->access_token  = $response->access_token;
        $dbTokenResponse->expires_in    = $response->expires_in;
        $dbTokenResponse->refresh_token = $response->refresh_token;
        update_option('argusDiscordOAuth2TokenResponse', serialize($dbTokenResponse));
        update_option('argusDiscordOAuth2AccessToken', $response->access_token);
        return true;
    }


    private static function prepareToLargeInsert ($table, $unique = true, $truncate = true) {
        global $wpdb;
        if ($truncate) $wpdb->query('TRUNCATE TABLE `' . $wpdb->prefix . $table . '`;');
        $wpdb->query('SET AUTOCOMMIT = 0;');
        if ($unique) $wpdb->query('SET UNIQUE_CHECKS = 0;');
    }


    private static function restoreAfterLargeInsert () {
        global $wpdb;
        $wpdb->query('SET UNIQUE_CHECKS = 1;');
        $wpdb->query('SET AUTOCOMMIT = 1;');
    }


    static function getData ($key) {
        $argusDiscordData = unserialize(get_option('argusDiscordData' . $key));
        if (!$argusDiscordData) return false;
        return $argusDiscordData;
    }


    private static function getAccessKeys () {
        return [
            'argusDiscordOAuth2AccessToken' => get_option('argusDiscordOAuth2AccessToken'),
            'argusDiscordOAuth2BotToken'    => get_option('argusDiscordOAuth2BotToken')
        ];
    }


    static function interpretMessage ($message, $filter = ['customEmoji' => true, 'images' => true, 'links' => true, 'tenor' => true]):array {
        $new           = [];
        $new['avatar'] = Data::getAvatarLinkOfTheGuildMember($message->author, 64);
        $new['name']   = $message->author->username;
        $new['date']   = $message->timestamp;
        $content       = $message->content;

        // Discord formatting styles
        $content = nl2br($content);
        $content = preg_replace('/(.*)(\*\*\*\_\_{1})(.*)(\_\_\*\*\*{1})(.*)/', '$1<i><b><u>$3</u></b></i>$5', $content);
        $content = preg_replace('/(.*)(\*\*\*{1})(.*)(\*\*\*{1})(.*)/', '$1<i><b>$3</b></i>$5', $content);
        $content = preg_replace('/(.*)(\*\*{1})(.*)(\*\*{1})(.*)/', '$1<b>$3</b>$5', $content);
        $content = preg_replace('/(.*)(\*{1})(.*)(\*{1})(.*)/', '$1<i>$3</i>$5', $content);
        $content = preg_replace('/(.*)(\_\_{1})(.*)(\_\_{1})(.*)/', '$1<u>$3</u>$5', $content);
        $content = preg_replace('/(.*)(\~\~{1})(.*)(\~\~{1})(.*)/', '$1<s>$3</s>$5', $content);

        // Filtering links in all messages
        if (!$filter['links']) {
            $content = preg_replace('@(https?://([-\w\.]+[-\w])+(:\d+)?(/([\w/_\.#-]*(\?\S+)?[^\.\s])?).*$)@', '', $content);
        }

        // Filtering custom emoji
        if ($filter['customEmoji']) {
            $content = preg_replace('/<:([^:]{1,}):([^>]{1,})>/u', '<img class="argus_widget__custom_emoji" src="https://cdn.discordapp.com/emojis/$2.png">', $content);
        } else {
            $content = wp_strip_all_tags($message->content);
        }

        // Filtering tenor messages
        if (strpos($message->content, 'tenor.com') !== false) {
            $tenor = $message->content;
            if (!$filter['links']) {
                $tenor = '';
            }
            if ($filter['tenor']) {
                $tenor = '<video controls loop autoplay muted><source src="' . $message->embeds[0]->video->url . '" type="video/mp4"></video>';
            }
            $content = $tenor;
        }

        // Filtering messages with attachments
        if (!empty($message->attachments)) {
            foreach ($message->attachments as $attachment) {
                if ($filter['images'] && preg_match('/(\.jpeg|\.jpg|.png|.gif)+$/', $attachment->url)) {
                    $spacer = '';
                    if ($content) {
                        $spacer = '<span class="argus_widget__content_spacer"></span>';
                    }
                    $content .= $spacer . '<img src="' . $attachment->url . '" >';
                } elseif ($filter['links']) {
                    $space = '';
                    if ($content) {
                        $space = '&nbsp;';
                    }
                    $content .= $space . $attachment->url;
                }
            }
        }

        $new['content'] = $content;
        $new['fetched'] = $message;
        return $new;
    }


}
