<?php

namespace Argus\Discord;

use Elementor\Controls_Manager;
use Elementor\Widget_Base;


class ElementorWidgetConnectProfile extends Widget_Base {


    function get_name ():string {
        return 'argus_discord_connect_profile_widget';
    }


    function get_title ():string {
        return __("Connect Discord Profile Button", 'argus-wordpress-plugin');
    }


    function get_icon ():string {
        return 'argus-discord-icon-connect-profile-button';
    }


    function get_categories ():array {
        return ['argus-discord-category'];
    }


    protected function register_controls () {

        $this->start_controls_section('section_content', [
            'label' => __("Settings", 'argus-wordpress-plugin'),
            'tab'   => Controls_Manager::TAB_CONTENT,
        ]);

        $this->add_control('connect_text', [
            'label'   => __("Connect text", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::TEXT,
            'default' => __("Connect Discord Profile", 'argus-wordpress-plugin'),
        ]);

        $this->add_control('disconnect_text', [
            'label'   => __("Disconnect text", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::TEXT,
            'default' => __("Disconnect Discord Profile", 'argus-wordpress-plugin'),
        ]);

        $this->end_controls_section();

        $this->start_controls_section('section_style', [
            'label' => __("Settings", 'argus-wordpress-plugin'),
            'tab'   => Controls_Manager::TAB_STYLE,
        ]);

        $this->add_control('style', [
            'label'   => __("Style", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::SELECT,
            'default' => 'no',
            'options' => [
                'no'      => __("None", 'argus-wordpress-plugin'),
                'default' => __("Default", 'argus-wordpress-plugin'),
            ],
        ]);

        $this->end_controls_section();
    }


    protected function render () {
        $userID = get_current_user_id();
        if (!$userID) return '';

        $settings = $this->get_settings_for_display();

        $discordID = get_user_meta($userID, 'dwpp-discord-id', true);

        if ($settings['style'] != 'no') wp_enqueue_style('argus-discord-front-end-styles');

        if ($discordID) {
            $text = $settings['disconnect_text'] ?? __("Disconnect Discord Profile", 'argus-wordpress-plugin');
        } else {
            $text = $settings['connect_text'] ?? __("Connect Discord Profile", 'argus-wordpress-plugin');
        }

        if ($discordID) {
            $link = '/?state=argus-disconnect-discord-profile&rand=' . time();
        } else {
            $link = Data::getAuthorizationLink('argus-connect-discord-profile-to-user');
        }

        if (isset($settings['style'])) {
            $class = 'argus-discord__button--' . $settings['style'] . '-style';
        } else {
            $class = 'argus-discord__button--no-style';
        }

        ?>
        <a class="<?php echo $class; ?>" href="<?php echo $link; ?>"><?php echo $text; ?></a>
        <?php
    }


    protected function content_template () {
        ?>
        <a class="argus-discord__button--{{{ settings.style }}}-style" href="javascript:void(0)">
            {{{ settings.disconnect_text ? settings.disconnect_text : "<?php _e("Disconnect Discord Profile", 'argus-wordpress-plugin'); ?>" }}}
        </a>
        <?php
    }


}
