<?php

namespace Argus\Discord;

use Elementor\Controls_Manager;
use Elementor\Widget_Base;


class ElementorWidgetSynchronization extends Widget_Base {


    function get_name ():string {
        return 'argus_discord_synchronization_widget';
    }


    function get_title ():string {
        return __("Synchronize Discord Profile Button", 'argus-wordpress-plugin');
    }


    function get_icon ():string {
        return 'argus-discord-icon-synchronize-profile-button';
    }


    function get_categories ():array {
        return ['argus-discord-category'];
    }


    protected function register_controls () {

        $this->start_controls_section('section_content', [
            'label' => __("Main", 'argus-wordpress-plugin'),
            'tab'   => Controls_Manager::TAB_CONTENT,
        ]);

        $this->add_control('text', [
            'label'   => __("Button text", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::TEXT,
            'default' => __("Synchronize Data", 'argus-wordpress-plugin'),
        ]);

        $this->end_controls_section();

        $this->start_controls_section('section_style', [
            'label' => __("Settings", 'argus-wordpress-plugin'),
            'tab'   => Controls_Manager::TAB_STYLE,
        ]);

        $this->add_control('style', [
            'label'   => __("Style", 'argus-wordpress-plugin'),
            'type'    => Controls_Manager::SELECT,
            'default' => 'no',
            'options' => [
                'no'      => __("None", 'argus-wordpress-plugin'),
                'default' => __("Default", 'argus-wordpress-plugin'),
            ],
        ]);

        $this->end_controls_section();
    }


    protected function render () {
        $wpUserID = get_current_user_id();
        if (!$wpUserID || !get_user_meta($wpUserID, 'dwpp-discord-id', true)) {
            return '';
        }

        $settings = $this->get_settings_for_display();

        if ($settings['style'] != 'no') wp_enqueue_style('argus-discord-front-end-styles');

        if (isset($settings['style'])) {
            $class = 'argus-discord__button--' . $settings['style'] . '-style';
        } else {
            $class = 'argus-discord__button--no-style';
        }

        ?>
        <a class="<?php echo $class; ?>" href="/?action=argus-synchronize-user&rand=<?php echo time(); ?>"><?php echo $settings['text']; ?></a>
        <?php
    }


    protected function content_template () {
        ?>
        <a class="argus-discord__button--{{{ settings.style }}}-style" href="javascript:void(0)">
            {{{ settings.text ? settings.text : "<?php _e("Synchronize Data", 'argus-wordpress-plugin'); ?>" }}}
        </a>
        <?php
    }


}
