<?php

namespace Argus;

class Data extends API {

    private static string $expireSupportDateFetchInterval = '-3 days';


    static function checkPurchaseCode ($purchaseCode) {
        $result = self::requestPurchaseCodeCheck($purchaseCode);

        if (isset($result->success) && $result->success) {
            update_option('argusDomainAuthentication', 1);
            update_option('argusPurchaseCode', $purchaseCode);
            $result->support = self::getSupportExpireDate();
        }

        return $result;
    }


    static function getSupportExpireDate () {
        $currentTimestamp = time();
        $expireDate       = get_option('argusSupportExpireDate');
        $lastUpdate       = get_option('argusSupportExpireDateLastUpdate');

        if (!$lastUpdate || !$expireDate || $lastUpdate < strtotime(self::$expireSupportDateFetchInterval)) {
            $result = self::requestGetSupportExpireDate(get_option('argusPurchaseCode'));

            if (isset($result->success) && $result->success) {
                $expireDate = strtotime($result->message);
                update_option('argusSupportExpireDate', $expireDate);
                update_option('argusSupportExpireDateLastUpdate', $currentTimestamp);
            }
        }

        return $expireDate ? date(get_option('date_format'), $expireDate) : '';
    }


    static function getDownloadURL () {
        $check = self::checkThePurchaseCode();
        if ($check->success) {
            $result = self::requestDownloadURL(get_option('argusPurchaseCode'));
            if (isset($result->success) && $result->success) {
                return $result->message;
            }
        }
        return false;
    }


    static function getLastUpdateInfo () {
        $check = self::checkThePurchaseCode();
        if ($check->success) {
            $version = self::requestLastVersion(get_option('argusPurchaseCode'));
            update_option('argusLastUpdateOnServer', $version);
            return $version;
        } else return $check;
    }


    private static function checkThePurchaseCode ():object {
        $purchaseCode = get_option('argusPurchaseCode');
        if ($purchaseCode) {
            return (object)['success' => true];
        } else {
            return (object)[
                'success' => false,
                'message' => __("Purchase code not found", 'argus-wordpress-plugin')
            ];
        }
    }


    static function saveURLForRedirect () {
        ?>
        <script>
            document.cookie = "argusLastPage=<?php echo $_SERVER['REQUEST_URI'];?>; expires=0; path=/";
            <?php if ( isset($_COOKIE['argusLastPage']) && $_COOKIE['argusLastPage'] != $_SERVER['REQUEST_URI'] ) { ?>
            document.cookie = "argusPrevPage=<?php echo $_COOKIE['argusLastPage'];?>; expires=0; path=/";
            <?php } ?>
        </script>
        <?php
    }


    static function unwrapArrayInString ($data, $string = '') {
        if (is_array($data)) {
            foreach ($data as $key => $record) {
                if (is_string($record)) {
                    $string .= $key . ': ' . $record . ' ';
                } elseif (is_array($record)) {
                    $string .= $key . ': ' . self::unwrapArrayInString($record, $string);
                }
            }
        }
        return $string;
    }


}
