<?php

namespace Argus;

use DateTime;


/*
 *
 * Component ID's for argus_log.component column
 *
 * 0 - General plugin's record - namespace Argus
 * 1 - Discord                 - namespace Argus\Discord
 * 2 - Steam                   - namespace Argus\Steam
 *
 *
 * Logging error levels for argus_log.level column
 *
 * 10 - Information - Useful information related to the operation of the system.
 * 20 - Warning     - Significant events indicate an error, but the behavior is either expected or not critical.
 * 30 - Error       - Log entries that indicate a system error that may be critical.
 * 40 - Fatal Error - Any error that is forcing a shutdown of the service or application.
 *
 */



class Log {

    static function getComponentLogs ($component = 0) {
        global $wpdb;
        $timestampFormat = get_option('date_format') . ' ' . get_option('time_format');

        $logs = $wpdb->get_results($wpdb->prepare("SELECT `level`, `timestamp`, `text` FROM `{$wpdb->prefix}argus_log` WHERE `component` = '%d' ORDER BY `timestamp` DESC;", $component));
        foreach ($logs as $record) {
            $record->timestamp = (new DateTime($record->timestamp))->format($timestampFormat);
        }

        return $logs;
    }


    static function countComponentCriticalErrors ($component = 0) {
        global $wpdb;
        $data = $wpdb->get_row($wpdb->prepare("SELECT COUNT(*) AS `count` FROM `{$wpdb->prefix}argus_log` WHERE `component`='%d' AND (`level`=30 OR `level`=40) LIMIT 1;", $component));
        return $data->count;
    }


    static function addMessage ($message, $component = 0, $level = 30) {
        global $wpdb;
        $wpdb->hide_errors();
        $insertStatus = $wpdb->insert($wpdb->prefix . 'argus_log', [
            'component' => $component,
            'level'     => $level,
            'text'      => $message,
        ], ['%d', '%d', '%s']);
        $wpdb->show_errors();
        return $insertStatus;
    }


    static function cleanLogs ($delay = false):void {
        global $wpdb;
        $delayQuery = '';
        if ($delay) $delayQuery = ' - INTERVAL 7 DAY';
        $wpdb->query("DELETE FROM `{$wpdb->prefix}argus_log` WHERE `timestamp` < NOW()$delayQuery;");
    }

}
